"""

    Connection.py
    $Author: payne $
    $Date: 2002/04/18 19:18:42 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

__version__ = '$Revision: 1.22 $'[11:-2]

import re
from string import join

from LDAPExceptions import *
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists

class AnonymousUser:
    """
        Encapsulates the actions that any user of the STScI
        Master Directory can do without authenticating
        to the directory.
    """

    def __init__(self, ldapDirectory):
        """
            Initializes the object and creates a connection to 
            the LDAP server.  Assigns the user the ANON role.
        """
        self.distinguishedName = ""
        self.LDAPDirectory = ldapDirectory

        self.bindAnonymously(self.LDAPDirectory.getServer(),
                             self.LDAPDirectory.getPort())

        self._roles = ['ANON',]

    def bindAnonymously(self, server, port):
        """
            Opens a connection to the LDAP server and
            performs an anonymous bind to the directory.
            The connection object is assigned to the
            'Connection' attribute of self.
        """
        try:
            self.Connection = _ldap.open(server, port)
        except _ldap.LDAPError, e:
            raise OPEN_FAILURE(source=e,
                               information="server = %s:%s" % ( server, port))
        except Exception, e:
            raise LDAPException(information="Server: %s, Port: %s" % \
                                (server, str(port)))
        try:
            self.Connection.simple_bind_s('','')
        except _ldap.LDAPError, e:
            raise BIND_FAILURE(source=e)

    def getAllGroups( self, searchBase= ""):
        """
            Returns a list of distinguishedNames of all group-like objects in
            or below the searchBase.
        """
        if searchBase == "":
            searchBase = self.LDAPDirectory.getDirectoryBase()

        query = '(|(objectClass=groupofnames)(objectClass=groupofuniquenames))'
        try:
            groups = self.search(searchBase, _ldap.SCOPE_SUBTREE, query, ['cn'])
            return [item[0] for item in groups]
        except NO_SUCH_OBJECT:
            return []

    def getMatchingGroups( self, groupName, searchBase=""):
        """
            Returns a list of distinguishedNames of all group-like objects 
            in or below the searchBase, whose naming attribute matches
            (case insensitive) groupName.
        """
        return [item for item in self.getAllGroups( searchBase )
                if getNameFromDN(item).lower() == groupName.lower()]

    def _getChildren(self, parentDN, query ):
        """
            Returns a list of all of the immediate children
            of parentDN that satisfy the query.
        """
        try:
            list = self.search(parentDN,
                                 _ldap.SCOPE_ONELEVEL,
                                 query,
                                 ['objectClass'])
            transformedList = [MappingObject(item[0]) for item in list]
            transformedList.sort(self._nameSort)
            return transformedList
        except NO_SUCH_OBJECT:
            return []

    def getChildGroups( self, parentDN ):
        """
            Returns all of the groups that are immediate children
            of the parentDN.
        """
        query = '(|(objectClass=groupofnames)(objectClass=groupofuniquenames))'
        return self._getChildren(parentDN, query)

    def getChildOrgUnits( self, parentDN ):
        """
            Returns all of the organizational units that are
            immediate children of the parentDN.
        """
        query = '(objectClass=organizationalunit)'
        return self._getChildren(parentDN, query)
    

    def getDNfromUserName( self, userName ):
        """
            Give a userName, returns the distinguishedName for that user.
            If there is no matching user in the directory, raises a
            NO_SUCH_OBJECT exception.
        """
        user = self.search( self.LDAPDirectory.getDirectoryBase(),
                            _ldap.SCOPE_SUBTREE,
                            join(['uid=', userName], '') )
        return user[0][0]  

    def getLDAPDirectory( self ):
        """
            Returns the LDAPDirectory singleton.
        """
        return self.LDAPDirectory

    def getNameFromDN( self, distinguishedName ):
        """
            Wrapper for STLDAPUtilities.getNameFromDN
        """
        return getNameFromDN( distinguishedName )

    def getRoles(self):
        """
            Returns a list of roles that the current user holds.
        """
        return self._roles

    def _nameSort( self, x, y ):
        """
            Helper function for sorting MappingObjects
        """
        a = x.mappingName
        b = y.mappingName
        return cmp( a, b )

    def search(self, base, scope, query, attrList = None):
        """
            Performs a generic search of the LDAP directory and returns the
            result.  If the search does not find anything, raises a
            NO_SUCH_OBJECT exception.

            Returns a sequence of results.  The sequence items are themselves
            a sequence containing the items distinguished name and a dictionary
            of attribute name/value pairs.  Each value item is itself a sequence
            of strings.
        """
        try:
            result = self.Connection.search_s(base, scope, query, attrList)
        except _ldap.LDAPError, e:
            raise NO_SUCH_OBJECT(source=e,
                                 information='Searchbase: %s, Query: %s' %(base, query))
        if result:
            return result
        else:
            raise NO_SUCH_OBJECT(information='Searchbase: %s, Query: %s' %(base, query))

    def unbind(self):
        """
            Closes the connection to the directory.
        """
        try:
            self.Connection.unbind()
        except _ldap.LDAPError, e:
            raise LDAPException(source=e, information='Unbind failed.')

class AuthenticatedUser(AnonymousUser):
    """
        Encasulates the actions that a user can perform after
        authenticating to the STScI Master Directory in addition
        to those actions that an anonymous user can perform.
    """

    def __init__(self, userName, passwd, ldapDirectory):
        """
            Attempts to authenticate to the directory with the
            supplied credentials.  If that authentication succeeds,
            determines which roles this user holds and sets the
            other attributes of the authenticated user.
        """

        AnonymousUser.__init__(self, ldapDirectory)

        self.distinguishedName = self.getDNfromUserName(userName)

        try:
            self.Connection.simple_bind_s(self.distinguishedName, passwd)
        except _ldap.LDAPError, e:
            raise INVALID_CREDENTIALS(source = e,
                                      information = 'Username: %s, DN: %s'
                                      % (userName, self.distinguishedName))

        user = self.search( self.distinguishedName, 
                            _ldap.SCOPE_BASE, 
                            join(['uid=', userName], ''))

        self.attributes = user[0][1]
        
        self._roles.append( 'AUTH' )
        self._roles = self._roles + self._getZopeRoles()

    def addRole(self, role):
        """
            Adds the given role to the user's list of roles.
        """
        if not role in self._roles:
            self._roles.append(role)
        
    def attribute(self, name):
        """
            Returns sequence containing the values of the user's
            attribute named 'name'.  Returns None if that attribute
            either is invalid or is not populated.
        """
        if self.attributes.has_key(name):
            return self.attributes[name]
        else:
            return None

    def _getZopeRoles(self):
        """
            Returns a list of all of the configured Zope Role groups that this
            user is a member of.
        """
        configuredRoles = \
            self.LDAPDirectory.getManagedRoles().getConfiguredRoleNames()

        query = \
            "(&(|(objectClass=groupOfNames)(objectClass=groupOfUniqueNames))" +\
            "(|(member=%s)(uniqueMember=%s)))" \
              % (self.distinguishedName, self.distinguishedName)
        try:
            roles = self.search(self.LDAPDirectory.getZopeRoleSearchBase(),
                                self.LDAPDirectory.getZopeRoleSearchScope(),
                                query,
                                ['cn'])
            return [getNameFromDN(item[0]) for item in roles if 
                    getNameFromDN(item[0]) in configuredRoles]
        except NO_SUCH_OBJECT:
            return []

    def rename(self, currentDN, newRDN ):
        """
            Renames an object (or, more correctly, changes an object's relative
            distinguished name).
        """
        try:
            self.Connection.modrdn_s(currentDN, newRDN)
        except _ldap.LDAPError, e:
            raise RENAME_ERROR(source = e,
                               information = currentDN )

class MappingObject:
    """
        This object is defined as a convenience.  By having getChildGroups and
        getChildOrgUnits return a list of MappingObjects instead of a list of
        dictionaries, it makes it easier to use the Zope tree tag.
    """
    def __init__(self, mappingDN):
        self.mappingDN = mappingDN
        self.mappingName = getNameFromDN(mappingDN)

    def __repr__(self):
        return '[' + self.mappingDN + ': ' + self.mappingName + ']'
