"""

    LDAPDirectory.py
    $Author: payne $
    $Date: 2002/01/03 20:21:12 

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    The LDAPDirectory object holds all of the information about the
    LDAP directory that gets persisted.  It holds the hostName:port,
    the DN of the directory search root, the DN of the Zope Role 
    group search, and the scope of the Zope role group search.  It
    also holds all of the the information about the directory schema,
    storing it in three ManagedSchema objects, one for each of 'user',
    'orgUnit', and 'group' types of objects.  And finally, it holds
    a ManagedRoles object, which contains the information about roles.

    The class is a singleton, and is initialized only when instantiating
    a new LDAP manager, in the init method of that object.

"""


__version__ = '$Revision: 1.13 $'[11:-2]

from urllib import splitnport

from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from SchemaDefaults import *
from STLDAPUtilities import _ldap

class LDAPDirectory:
    """
        This class is a singleton and is persisted by Zope.
        It abstracts all of the properties of the LDAP directory
        server.
    """

    def __init__(self,
                 ldapServer,
                 ldapDirectoryBase,
                 personSearchBase = '',
                 orgUnitBrowseBase = '',
                 groupBrowseBase = '',
                 zopeRoleSearchBase = '',
                 zopeRoleSearchScope = _ldap.SCOPE_SUBTREE ):

        self.setServer( ldapServer )
        self.setDirectoryBase( ldapDirectoryBase )
        if personSearchBase:
            self.setPersonSearchBase( personSearchBase )
        else:
            self.setPersonSearchBase( ldapDirectoryBase )

        if orgUnitBrowseBase:
            self.setOrgUnitBrowseBase( orgUnitBrowseBase )
        else:
            self.setOrgUnitBrowseBase( ldapDirectoryBase )

        if groupBrowseBase:
            self.setGroupBrowseBase( groupBrowseBase )
        else:
            self.setGroupBrowseBase( ldapDirectoryBase )

        if zopeRoleSearchBase:
            self.setZopeRoleSearchBase( zopeRoleSearchBase )
        else:
            self.setZopeRoleSearchBase( ldapDirectoryBase )

        self.setZopeRoleSearchScope ( zopeRoleSearchScope )

        self._groupManagedSchema = \
                ManagedSchema(defaultGroupSchema,
                              defaultGroupDisplayCategories,
                              defaultGroupNamingAttribute,
                              defaultGroupControllers)

        self._orgUnitManagedSchema = \
                ManagedSchema(defaultOrgUnitSchema,
                              defaultOrgUnitDisplayCategories,
                              defaultOrgUnitNamingAttribute,
                              defaultOrgUnitControllers)

        self._personManagedSchema = \
                ManagedSchema(defaultUserSchema,
                              defaultUserDisplayCategories,
                              defaultUserNamingAttribute,
                              defaultUserControllers)

        self._managedRoles = ManagedRoles()

    def getDirectoryBase(self):
        """
            Returns the distinguished name of the top of the
            LDAP directory tree.
        """
        return self._ldapDirectoryBase

    def getGroupSchema(self):
        """
            Returns the managedSchema object for LDAP Groups.
        """
        return self._groupManagedSchema

    def getGroupBrowseBase(self):
        """
            Returns the configured group browse base.
        """
        return self._groupBrowseBase

    def getManagedRoles(self):
        """
            Returns the managedRoles object for the Zope/directory
            interface.
        """
        return self._managedRoles

    def getOrgUnitBrowseBase(self):
        """
            Returns the configured orgUnit browse base.
        """
        return self._orgUnitBrowseBase
   
    def getOrgUnitSchema(self):
        """
            Returns the managedSchema object for LDAP Org. Units.
        """
        return self._orgUnitManagedSchema
   
    def getPersonSchema(self):
        """
            Returns the managedSchema object for LDAP persons.
        """
        return self._personManagedSchema
   
    def getPersonSearchBase(self):
        """
            Returns the configured personSearchBase.
        """
        return self._personSearchBase

    def getPort(self):
        """
            Returns the IP Port number that the LDAP
            directory server listens on.
        """
        return self._ldapPort

    def getServer(self):
        """
            Returns the IP name of the LDAP Directory server
        """
        return self._ldapServer

    def getZopeRoleSearchBase(self):
        """
            Returns the distinguished name of the container
            that contains LDAP group objects that are to
            be identified with Zope roles.
        """
        return self._zopeRoleSearchBase

    def getZopeRoleSearchScope(self):
        """
            Returns the search scope for finding LDAP group
            objects that are to be identified with Zope
            roles.
        """
        return self._zopeRoleSearchScope
                 
    def setDirectoryBase(self, directoryBase ):
        """
            Sets the distinguished name of the top of the
            LDAP directory tree.
        """
        self._ldapDirectoryBase = directoryBase

    def setGroupBrowseBase( self, groupBrowseBase ):
        """
            Sets the distinguished name of the root object for
            browsing "group" objects.  Something like
            "ou=groups,o=my.org"
        """
        self._groupBrowseBase = groupBrowseBase

    def setOrgUnitBrowseBase( self, orgUnitBrowseBase ):
        """
            Sets the distinguished name of the root object for
            browsing "Org Unit" objects.  If part of your directory
            mimics your org chart, point this to it.
        """
        self._orgUnitBrowseBase = orgUnitBrowseBase

    def setPersonSearchBase( self, personSearchBase ):
        """
            Sets the distinguished name of the root object for
            "person" objects.  Usually something like
            "ou=people,o=my.org".
        """
        self._personSearchBase = personSearchBase

    def setServer(self, ldapServer ):
        """
            Sets the IP name of the LDAP Directory server, and
            port number.
        """
        self._ldapServer,self._ldapPort = splitnport( ldapServer, 389 )

    def setZopeRoleSearchBase( self, zopeRoleSearchBase ):
        """
            Sets the distinguished name of the container
            that contains LDAP group objects that are to
            be identified with Zope roles.
        """
        self._zopeRoleSearchBase = zopeRoleSearchBase

    def setZopeRoleSearchScope( self, zopeRoleSearchScope ):
        """
            Sets the search scope for finding LDAP group
            objects that are to be identified with Zope
            roles.
        """
        self._zopeRoleSearchScope = zopeRoleSearchScope
                 
