"""

    LDAPObject.py
    $Author: payne $
    $Date: 2002/01/03 20:21:12 

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

__version__ = '$Revision: 1.30 $'[11:-2]

from LDAPExceptions import *
from ManagedAttributeSchema import ManagedAttribute
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists, TRUE, FALSE


class LDAPObject:
    """
        ldapObject is the base class for the LDAPGroup,
        LDAPOrgUnit, and LDAPPerson classes.
        It provides the functionality that is common to
        all those classes.  No instance of the ldapObject
        class should ever be created.
    """
    
    def __init__( self, user ):
        """
            Either a parentDN and objectName or a distinguishedName
            must be supplied.

            Retrieves the group information from the LDAP server and
            initializes the python object instance.

            If the group does not exist on the server and
                createGroup is true,
                    creates a new group.
                else
                    raises a NO_SUCH_OBJECT exception
        """

        self.user = user
        self.Connection = user.Connection
        self.searchBase = user.LDAPDirectory.getDirectoryBase()
        self.personSearchBase = user.LDAPDirectory.getPersonSearchBase()
        self.orgUnitBrowseBase = user.LDAPDirectory.getOrgUnitBrowseBase()
        self.groupBrowseBase = user.LDAPDirectory.getGroupBrowseBase()
        self.managedRoles = user.LDAPDirectory.getManagedRoles()
        self.distinguishedName = ""
        
    def canReadAttribute(self, attributeName):
        """
            Returns true if the current user has read access to
            the named attribute based on the roles the user holds.
        """
        readers = self._ManagedAttributes[attributeName].getReaders()
        for role in self.user.getRoles():
            if role in readers:
                return TRUE
        return FALSE
 
    def canWriteAttribute(self, attributeName):
        """
            Returns true if the current user has write access to
            the named attribute based on the roles the user holds.
        """
        writers = self._ManagedAttributes[attributeName].getWriters()
        for role in self.user.getRoles():
            if role in writers:
                return TRUE
        return FALSE

    def create(self, distinguishedName, objectCategory = 'None'):
        """
            Creates a new object of the appropriate type.  Adds defaults
            for all required attributes except objectClass, the naming attribute,
            and attributes which have required values.
        """
        objectName = getNameFromDN(distinguishedName)

        if objectCategory.lower() == 'group':
            self.managedSchema = self.user.LDAPDirectory.getGroupSchema()
        elif objectCategory.lower() == 'orgunit':
            self.managedSchema = self.user.LDAPDirectory.getOrgUnitSchema()
        elif objectCategory.lower() == 'person':
            self.managedSchema = self.user.LDAPDirectory.getPersonSchema()
        else:
            raise UNRECOGNIZED_OBJECT
    
        attrList = []

        for attrName in self.managedSchema.getRequiredAttributeNames():
            if attrName == 'objectClass':
                attrList.append( ('objectClass', self.getRequiredObjectClasses() ) )
            elif attrName == self.managedSchema.getNamingAttribute():
                attrList.append( (attrName, [objectName]) )
            elif attrName == 'member' or attrName == 'uniqueMember':
                # since OpenLDAP requires these attributes on a 
                # groupofname or groupofuniquenames, respectively
                attr = self.managedSchema.getAttributeSchema(attrName)
                values = attr.getRequiredValues()
                if values == []:
                    values = ['cn=Default Value']
                attrList.append( (attrName, values) )
            else:
                attr = self.managedSchema.getAttributeSchema(attrName)
                values = attr.getRequiredValues()
                if values == []:
                    values = ['Default Value']
                attrList.append( (attrName, values) )
                
        self.Connection.add_s(distinguishedName, attrList)
        self.refresh(distinguishedName)
        return objectName
        
    def delete( self, areYouSure = FALSE ):
        """
            Deletes the object object from the LDAP directory
            and invalidated the corresponding Python instance.
        """
        if areYouSure:
            self._deleteObject( self.distinguishedName )
            self.distinguishedName = None
            self.allowedAttributes = None


    def _deleteObject( self, distinguishedName ):
        """
            Recursively deletes all of the children of the current
            object and the deletes the object.
        """
        try:
            children = self.user.search( distinguishedName,
                                         _ldap.SCOPE_ONELEVEL,
                                         '(objectClass=*)',
                                         ['objectClass'] )
            for child in children:
                self._deleteObject( child[0] )
        except NO_SUCH_OBJECT:
            pass

        #   Before we delete this object, we should scan the
        #   entire directory looking for objects that have
        #   attributes of syntax 'DN' and that contain entries
        #   that point to this object
        
        try:
            self.Connection.delete_s( distinguishedName )
        except _ldap.LDAPError, e:
            raise DELETE_FAILURE(source=e,
                                 information = 'DN = %s' % (self.distinguishedName))

    def displayCategoryHasReadableAttributes( self, displayCategory ):
        """
            Returns TRUE or FALSE depending on whether the given
            display category contains any items the user can read.
        """
        for aName in self._ManagedAttributes.keys():
            if self._ManagedAttributes[aName].getManaged() \
            and displayCategory in self._ManagedAttributes[aName].getDisplayCategories() \
            and self.canReadAttribute(aName):
                return TRUE
        return FALSE

    def displayCategoryHasWritableAttributes( self, displayCategory ):
        """
            Returns TRUE or FALSE depending on whether the given
            display category contains any items the user can write.
        """
        for aName in self._ManagedAttributes.keys():
            if self._ManagedAttributes[aName].getManaged() \
            and displayCategory in self._ManagedAttributes[aName].getDisplayCategories() \
            and self.canWriteAttribute(aName):
                return TRUE
        return FALSE

    def friendlyNameSort( self, x, y ):
        """
            Helper function for getAttributes.
        """
        x = self._ManagedAttributes[x].getFriendlyName( )
        y = self._ManagedAttributes[y].getFriendlyName( )
        return cmp( x, y )

    def getNameFromDN( self, distinguishedName, REQUEST=None ):
        """
            Wrapper for STLDAPUtilities.getNameFromDN
        """
        return getNameFromDN( distinguishedName )

    def getRequiredObjectClasses(self):
        """
            Returns a list of objectClasses that the schema requires
            be in the objectClass attribute for this object.
        """
        oClassSchema = self.managedSchema.getAttributeSchema('objectClass')
        return oClassSchema.getRequiredValues()

    def getManagedAttribute( self, attributeName ):
        """
            Returns the  ManagedAttribute oject for the named attribute.
        """
        if not self._ManagedAttributes.has_key(attributeName):
            raise NO_SUCH_ATTRIBUTE(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        elif not self._ManagedAttributes[attributeName].getManaged():
            raise INSUFFICIENT_ACCESS(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        elif not self.canReadAttribute(attributeName):
            raise INSUFFICIENT_ACCESS(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        else:
            return self._ManagedAttributes[attributeName]
        

    def getManagedAttributeNames(self, displayCategory):
        """
            Returns a list of managed attribute names in the given
            display category and for which the user has at least
            read access.
        """
        names = [aName for aName in self._ManagedAttributes.keys()
                       if  self._ManagedAttributes[aName].getManaged()
                       and displayCategory in self._ManagedAttributes[aName].getDisplayCategories()
                       and self.canReadAttribute(aName)]
        names.sort(self.friendlyNameSort)
        return names

    def getObjectCategory(self):
        """
            Returns one of 'group', 'orgunit', 'person', defining the type of
            this object, or 'None' if the object has not been refreshed.
        """
        if hasattr( self, 'distinguishedName' ):
            if hasattr( self, '_objectCategory' ):
                return self._objectCategory
        else:
            return 'None'

    def refresh( self, distinguishedName = None ):
        """
            Reload the allowed attributes for the current
            instance from the underlying LDAP directory.
        """

        if self.distinguishedName != '':
            distinguishedName = self.distinguishedName

        if distinguishedName is None:
            raise NOT_ENOUGH_INFORMATION

        retrievedObject = self.user.search(distinguishedName,
                                           _ldap.SCOPE_BASE,
                                           '(objectclass=*)')

        self._dirty = FALSE
        self.distinguishedName = retrievedObject[0][0]
        self.parentDN = self.distinguishedName[self.distinguishedName.find(',')+1:]

        #   Really seeing that the user is not an AnonymousUser:

        if self.user.distinguishedName != "":
            if DNsAreEquivalent( self.distinguishedName, 
                                 self.user.distinguishedName ):
                self.user.addRole('SELF')
            
        retrievedAttributeValues = retrievedObject[0][1]

        oClass = [item.lower() for item in retrievedAttributeValues['objectClass']]

        if 'organizationalunit' in oClass:
            self.managedSchema = self.user.LDAPDirectory.getOrgUnitSchema()
            self._objectCategory = 'orgunit'
        elif 'person'in oClass:
            self.managedSchema = self.user.LDAPDirectory.getPersonSchema()
            self._objectCategory = 'person'
        elif ('groupofnames' in oClass) or ('groupofuniquenames' in oClass):
            self.managedSchema = self.user.LDAPDirectory.getGroupSchema()
            self._objectCategory = 'group'
        else:
            raise UNRECOGNIZED_OBJECT

        self._ManagedAttributes = {}
        
        for attributeName in self.managedSchema.getAllowedAttributeNames():
            
            metadata = self.managedSchema.getAttributeSchema(attributeName).getMetadata()

            values = retrievedAttributeValues.get(attributeName, [])
            attribute = ManagedAttribute(attributeName, values, metadata)
            self._ManagedAttributes[attributeName] = attribute

        if self._ManagedAttributes.has_key('manager'):
            mgrList =  self._ManagedAttributes['manager'].getValues()
            for mgr in mgrList:
                if DNsAreEquivalent(mgr, self.user.distinguishedName):
                    self.user.addRole('MNGR')
                    break

        return self.distinguishedName

    def rename( self, newObjectName ):
        """
            Renames the current object by changing it's rdn.
        """
        self.parentDN = self.distinguishedName[self.distinguishedName.find(',')+1:]
        newRDN = self.managedSchema.getNamingAttribute() + '=' + newObjectName
        try:
            self.Connection.modrdn_s( self.distinguishedName, newRDN )
        except _ldap.ALREADY_EXISTS:
            raise ALREADY_EXISTS
        
        self.distinguishedName = newRDN + ',' + self.parentDN
        return self.refresh()

    def setManagedAttribute( self, managedAttribute ):
        """
            Returns the  ManagedAttribute oject for the named attribute.
        """
        attributeName = managedAttribute.getAttributeName()
        
        if not self._ManagedAttributes.has_key(attributeName):
            raise NO_SUCH_ATTRIBUTE(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        elif not self._ManagedAttributes[attributeName].getManaged():
            raise INSUFFICIENT_ACCESS(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        elif not self.canWriteAttribute(attributeName):
            raise INSUFFICIENT_ACCESS(information = \
                                    'Attribute Name = %s' % \
                                    (attributeName))
        else:
            self._ManagedAttributes[attributeName] = managedAttribute
            if managedAttribute.ValueChanged():
                self._dirty = TRUE
        
    def updateDirectory( self ):
        """
            Writes any changes that have been made to this object to
            the LDAP directory.
        """
        changedAttrNames = [aName for aName in self._ManagedAttributes.keys()
                            if self._ManagedAttributes[aName].getManaged()
                            and self._ManagedAttributes[aName].ValueChanged()
                            and self.canReadAttribute(aName)]

        ldapModList = []

        for attrName in changedAttrNames:
            attrValues = self._ManagedAttributes[attrName].getValues()
            if attrValues == []:
                mode = _ldap.MOD_DELETE
            else:
                mode = _ldap.MOD_REPLACE
            ldapModList.append( ( mode, attrName, attrValues ) )

        if ldapModList != []:
            self.Connection.modify_s( self.distinguishedName, ldapModList )
            
        self._dirty = FALSE
