"""
    The LDAPSchema class.

    LDAPSchema.py
    $Author: payne $
    $Date: 2002/04/18 18:49:42 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.


    This class instantiates an iPlanet Directory Server schema,
    which can be used to find out about object classes known
    to the directory, and their attributes.  Instances of this
    class should not be made persistent.  Instead, a new 
    instance should be created as required, so as to reflect
    the current state of the directory.

    We need methods like those in the ManagedSchema class, at
    least the ones that allow us to query the schema.  But
    because there is only one LDAP schema, encompassing all
    of the objects that can be defined in the directory, and
    because objects in the directory can have more than one
    objectClass, we select the schema information we need by
    passing in a list of objectClasses, or, if that is not
    available, an objectCategory ('person', 'orgUnit', or 'group').

    Just a reminder:  'allowed' attributes are all of the 
    attributes defined in the directory for the given 
    object classes or category, 'required' attributes are those
    required in the schema, and 'optional' attributes are those
    that are allowed but not required.
"""

__version__ = '$Revision: 1.14 $'[11:-1]

from LDAPExceptions import *
from LDIFObjectParser import LDIFObjectParser
from LDIFAttributeParser import LDIFAttributeParser
from ManagedAttributeSchema import ManagedAttributeSchema
from STLDAPSyntax import getNameFromOID
from STLDAPUtilities import _ldap, TRUE, FALSE, listsAreEquivalent

class LDAPSchema:
    """
        A class to encapsulate an LDAP schema, obtained from the
        LDAP directory.
    """

    def __init__( self, user, objectClasses ):

        directoryBase = user.LDAPDirectory.getDirectoryBase()

        schemaDN = user.search( directoryBase, 
            _ldap.SCOPE_BASE, 
            "(objectclass=*)", 
            ['subschemaSubentry',])[0][1]['subschemaSubentry'][0]

        schema = user.search( schemaDN, _ldap.SCOPE_BASE,
            "(objectclass=*)", ['objectClasses','attributeTypes'] )[0][1]

        self._objectSchemas = \
                self._parseLDIFObjectDefinitions(schema['objectClasses'])
        self._attributeSchemas = \
                self._parseLIDFAttrDefinitions(schema['attributeTypes'])

        self.requiredAttributeNames, self.optionalAttributeNames \
                                     = self._buildAttributeLists(objectClasses)

        self.allowedAttributeNames = self.requiredAttributeNames \
                                   + self.optionalAttributeNames

        self._attributeCollection = \
            self._buildManagedAttributeCollection( objectClasses )

    def _buildAttributeLists(self, objectClasses):
        """
            Returns lists of required and optional attribute names
            for the composite LDAP object made up of all of the
            given objectClasses
        """
        optionalAttributeNames = []
        requiredAttributeNames = []

        if type(objectClasses) == type(''):
            objectClasses = [objectClasses]
            
        for objectClass in objectClasses:
            required, optional = self._getAttributeNames(objectClass)
            requiredAttributeNames = requiredAttributeNames \
                                   + [item for item in required \
                                      if not item in requiredAttributeNames]
            optionalAttributeNames = optionalAttributeNames \
                                    + [item for item in optional \
                                       if not item in optionalAttributeNames]
                                           
        optionalAttributeNames = [item for item in optionalAttributeNames \
                                  if not item in requiredAttributeNames ]

        return requiredAttributeNames, optionalAttributeNames

    def _buildManagedAttributeCollection( self, objectClasses ):
        """
            Returns a dictionary, keyed to attribute name and
            containing ManagedAttributeSchema objects for each
            allowed attribute for the objects in the objectClasses
            list.
        """
        attributeCollection = {}

        for attributeName in self.allowedAttributeNames:
            parsedAttribute = self._attributeSchemas[attributeName.lower()]
            metadata = {}
            metadata['syntax'] = parsedAttribute.syntax
            if attributeName in self.requiredAttributeNames:
                metadata['required'] = TRUE
            managedAttribute = ManagedAttributeSchema( attributeName, metadata )
            attributeCollection[ attributeName ] = managedAttribute

        return attributeCollection
    
    def _getAttributeNames(self, objectClass):
        """
            Returns lists of required and optional attributes for the
            given LDAP object class.
        """
        LDIFObject = self._objectSchemas[objectClass.lower()]
        required = LDIFObject.mustContain
        optional = LDIFObject.mayContain

        if LDIFObject.superClass:
            supRequired, supOptional = self._getAttributeNames(LDIFObject.superClass)
            required = required + [item for item in supRequired if not item in required]
            optional = optional + [item for item in supOptional if not item in optional]

        return required, optional            

    def getManagedAttributeCollection(self):
        """
            Returns a dictionary of ManagedAttributeSchema objects.
        """
        return self._attributeCollection.copy()

    def getManagedAttributeSchema( self, ldapName ):
        """ 
            Returns a named ManagedAttributeSchema object.
        """
        assert ldapName in self._attributeCollection.keys()
        return self._attributeCollection[ldapName]

    def _parseLIDFAttrDefinitions(self, LDIFExpressionList):
        """
            Parses a list of LDIF attribute definitions and returns a
            dictionary of LDIFAttributeParser objects.
        """
        attributeSchemas = {}
        for ldifExpression in LDIFExpressionList:
            parsedObject = LDIFAttributeParser( ldifExpression )
            attributeSchemas[parsedObject.name.lower()] = parsedObject
        return attributeSchemas
                   
    def _parseLDIFObjectDefinitions(self, LDIFExpressionList):
        """
            Parses a list of LDIF object definitions and returns a
            dictionary of LDIFObjectParser objects.
        """
        objectSchemas = {}
        for ldifExpression in LDIFExpressionList:
            parsedObject = LDIFObjectParser( ldifExpression )
            objectSchemas[parsedObject.name.lower()] = parsedObject
        return objectSchemas

    
