"""

    ManagedAttributeSchema.py
    $Author: payne $
    $Date: 2002/04/18 19:18:42 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    This module contains the attribute classes.  A base virtual
    VirtualAttributeSchema schema defines an object that holds
    an attribute schema: the metadata that completely describes
    a single attribute of an LDAP object, and methods for returning
    those metadata.  The ManagedAttributeSchema class subclasses
    the virtual class, and adds methods for changing the metadata.
    ManagedAttributeSchema class objects are used under the 'Schema'
    tab to save the schema configuration.  Finally, a parallel
    ManagedAttribute class subclasses the base virtual class, adding
    data values and methods to handle them.
"""

__version__ = '$Revision: 1.15 $'[11:-2]

import base64
import sha
from urllib import quote, quote_plus

from LDAPExceptions import *
import STLDAPSyntax
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists, TRUE, FALSE

metadataKeys = [ 'friendlyName',
                 'readers',
                 'writers',
                 'syntax',
                 'required',
                 'managed',
                 'validValues',
                 'requiredValues',
                 'displayCategories',
                 'pointsTo' ]

validPointsTo = ['person', 'group', 'orgunit', '']

class VirtualAttributeSchema:
    """
        This class provides a basis for the ManagedAttributeSchema
        and the ManagedAttribute classes.  It should never be
        instantiated on its own.
    """
    def __init__(self, name, metadata=None):
        
        self._ldapName = name
        
        if metadata:
            assert type(metadata) == type({})
            self._friendlyName = metadata.get('friendlyName', '')
            self._readers = metadata.get('readers', [])
            self._writers = metadata.get('writers', [])
            
            if metadata.has_key('syntax') and metadata['syntax']:
                if STLDAPSyntax.isValidOID(metadata['syntax']):
                    self._syntax = metadata['syntax']
                else:
                    self._syntax = STLDAPSyntax.getOIDFromName(metadata['syntax'])
            elif metadata.has_key('sup'):
                self._sup = metadata['sup']
                    
            self._managed = metadata.get('managed', FALSE)                    
            self._required = metadata.get('required', FALSE)
            self._validValues = metadata.get('validValues', [])
            self._requiredValues = metadata.get('requiredValues', [])
            self._displayCategories = metadata.get('displayCategories', [])
            self._pointsTo = metadata.get('pointsTo', '')
            
        self._metadataChanged = FALSE

    def getAttributeName(self):
        """
            Returns the LDAP name of this attribute.
        """
        return self._ldapName

    def getAvailableValues(self):
        """
            Returns a list of valid values which are not already required.
        """
        return [item for item in self.getValidValues()
                if item not in self.getRequiredValues()]

    def getDisplayCategories(self):
        """
            Returns a list of the display categories for
            this attribute.
        """
        return listCopy(getattr(self, '_displayCategories', []))

    def getFriendlyName(self):
        """
            Returns the attribute's friendly name.  If one is not defined,
            returns the LDAP name.
        """
        return getattr(self, '_friendlyName', self._ldapName)

    def getManaged(self):
        """
            Returns TRUE if we are actively managing this attribute,
            false otherwise.
        """
        return getattr(self, '_managed', FALSE)

    def getPointsTo(self):
        """
            Returns the type of object that this attribute points to.
            Needed only if this attribute is a distinguishedName.
        """
        return getattr(self, '_pointsTo', '')

    def getReaders(self):
        """
            Returns a list of roles that have
            permission to read the attribute.
        """
        return listCopy(getattr(self, '_readers', []))

    def getRequired(self):
        """
            Returns True if this attribute is required for
            the LDAP class that the attribute is part of.
        """
        return getattr(self, '_required', FALSE)

    def getRequiredValues(self):
        """
            Returns the list of required values for this attribute.
            Usually used only with the objectClass attribute.
        """
        return listCopy(getattr(self, '_requiredValues', []))

    def getSup(self):
        """
            Returns the name of the SUP attribute, if this attribute
            inherits from another attribute, or else the empty string.
        """
        return getattr(self, '_sup', '')

    def getSyntaxName(self):
        """
            Returns name of the LDAP syntax for this attribute.
        """
        oid = getattr(self, '_syntax', '')
        if oid:
            return STLDAPSyntax.getNameFromOID(oid)
        else:
            return ''

    def getSyntaxOID(self):
        """
            Returns the OID for the LDAP syntax of this attribute.
        """
        return getattr(self, '_syntax', '')

    def getValidValues(self):
        """
            Returns a list of valid values for the attribute.
        """
        return listCopy(getattr(self, '_validValues', []))

    def getWriters(self):
        """
            Returns a list of roles that have
            permission to modify the attribute.
        """
        return listCopy(getattr(self, '_writers', []))

    def hasValidValues(self):
        """
            Returns TRUE if the list of valid values contains
            non-trivial values.
        """
        if self.getValidValues() == []:
            return FALSE
        for item in self.getValidValues():
            if item.strip() != '':
                return TRUE
        return FALSE

    def inheritsFrom( self, obj ):
        """
            Inherits property values (e.g., syntax) from a superior
            ManagedAttribute object.
        """
        assert isinstance( obj, VirtualAttributeSchema )
        self._syntax = obj.getSyntaxOID()

class ManagedAttributeSchema(VirtualAttributeSchema):
    """
        Encapsulates the schema of an LDAP attribute.  This class
        inherits methods for reading the schema from the
        VirtualAttributeSchema class and adds methods for
        modifying that schema.
    """
        
    def __init__(self, name, metadata=None):

        VirtualAttributeSchema.__init__(self, name, metadata)              
        self._metadataChanged = FALSE

    def addDisplayCategories(self, values):
        """
            Merge the specified values with the display categories list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        self._displayCategories = mergeLists( self.getDisplayCategories(), 
                                              values )
        self._metadataChanged = TRUE

    def addReaders(self, values):
        """
            Merge the specified values with the readers list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        self._readers = mergeLists( self.getReaders(), 
                                    values )
        self._metadataChanged = TRUE

    def addRequiredValues(self, values):
        """
            Merge the specified values with the existing required
            values list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        if not list1ContainsList2(self.getRequiredValues(), values):
            self._metadataChanged = TRUE
            self._requiredValues = mergeLists( self.getRequiredValues(),values )

    def addValidValues(self, values):
        """
            Merge the specified values with the existing valid
            values list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        if not list1ContainsList2(self.getValidValues(), values):
            self._metadataChanged = TRUE
            self._validValues = mergeLists( self.getValidValues(), values )

    def addWriters(self, values):
        """
            Merge the specified values with the writers list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        self._writers = mergeLists( self.getWriters(), 
                                    values )
        self._metadataChanged = TRUE

    def delDisplayCategories(self, values):
        """
            Delete the specified values from the display categories list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        assert list1ContainsList2( self.getDisplayCategories(), values )
        for item in values:
            self._displayCategories.remove( item )
        self._metadataChanged = TRUE

    def delReaders(self, values):
        """
            Delete the specified values from the readers list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        for item in values:
            try:
                self._readers.remove( item )
            except:
                pass
        self._metadataChanged = TRUE

    def delRequiredValues(self, values):
        """
            Delete the specified values from the existing required
            values list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        assert list1ContainsList2( self.getRequiredValues(), values )
        for item in values:
            self._requiredValues.remove( item )
        self._metadataChanged = TRUE

    def delValidValues(self, values):
        """
            Delete the specified values from the existing valid
            values list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        assert list1ContainsList2( self.getValidValues(), values )
        for item in values:
            self._validValues.remove( item )
        self._metadataChanged = TRUE

    def delWriters(self, values):
        """
            Delete the specified values from the writers list.
        """
        if type(values) == type(''):
            values = [values,]
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        for item in values:
            try:
                self._writers.remove( item )
            except:
                pass
        self._metadataChanged = TRUE

    def getMetadata(self):
        """
            Returns a dictionary containing the metadata for this
            attribute.
        """
        tmp = {}
        for x in metadataKeys:
            if hasattr(self, '_' + x):
                tmp[x] = getattr(self, '_' + x)
        return tmp

    def MetadataChanged(self):
        """
            Returns TRUE if the properties of this attribute
            have changed since it was last saved.
        """
        return self._metadataChanged

    def setDisplayCategories(self, values):
        """
            Sets the list of display categories for this
            attribute.
        """
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        if not listsAreEquivalent(values, self.getDisplayCategories()):
            self._displayCategories = listCopy(values)
            self._metadataChanged = TRUE

    def setFriendlyName(self, value):
        """
            Sets the attribute's friendly name.
        """
        assert type(value) == type('')
        if value != self.getFriendlyName():
            self._friendlyName = value
            self._metadataChanged = TRUE

    def setManaged(self, value):
        assert type(value) == type(1)
        if value != self.getManaged():
            self._managed = value
            self._metadataChanged = TRUE

    def setPointsTo(self, value):
        """
            Sets the type of object that this attribute points to.
            Needed only if this attribute is a distinguishedName.
        """
        assert type(value) == type('')
        assert value.lower() in validPointsTo
        if value != self.getPointsTo():
            self._pointsTo = value.lower()
            self._metadataChanged = TRUE

    def setReaders(self, values):
        """
            Replaces the list of roles that have
            permission to read the attribute.
        """
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        if not listsAreEquivalent(values, self.getReaders()):
            self._metadataChanged = TRUE
            self._readers = listCopy(values)

    def setRequired(self, value):
        """
            Sets the required flag for this attribute.
        """
        assert type(value) == type(TRUE)
        if value != self._required:
            self._required = value
            self._metadataChanged = TRUE

    def setRequiredValues(self, values):
        """
            Sets the list of required values for this attribute.
            Usually used only with the objectClass attribute.
        """
        assert type(values) == type([])
        if not listsAreEquivalent(values, self.getRequiredValues()):
            self._requiredValues = listCopy(values)
            self._metadataChanged = TRUE

    def setSyntaxByName(self, value):
        """
            Sets the LDAP syntax for this attribute by name.
        """
        assert type(value) == type('')
        oid = STLDAPSyntax.getOIDFromName(value)
        self.setSyntaxByOID(oid)

    def setSyntaxByOID(self, value):
        """
            Sets the LDAP syntax for this attribute by OID.
        """
        assert STLDAPSyntax.isValidOID(value)
        if self.getSyntaxOID() != value:
            self._syntax = value
            self._metadataChanged = TRUE

    def setValidValues(self, values):
        """
            Sets the list of valid values for this attribute.
        """
        assert type(values) == type([])
        if not listsAreEquivalent(values, self.getValidValues()):
            self._metadataChanged = TRUE
            self._validValues = listCopy(values)

    def setWriters(self, values):
        """
            Replaces the list of roles that have
            permission to modify the attribute
        """
        assert type(values) == type([])
        values = [item for item in values if item.strip() != '']
        if not listsAreEquivalent(values, self.getWriters()):
            self._metadataChanged = TRUE
            self._writers = listCopy(values)

class ManagedAttribute(VirtualAttributeSchema):
    """
        Encapsulates all of the behavior of an LDAP attribute.
    """
        
    def __init__(self, name, values=[], metadata=None):
        
        VirtualAttributeSchema.__init__(self, name, metadata)
        self._value = values
        self._valueChanged = FALSE

    def addValue(self, value):
        """
            Adds a single value to the current list of values.
        """
        if not value in self.getValues():
            self._value.append(value)
            self._valueChanged = TRUE

    def delValue(self, value):
        """
            Deletes a single value from the current list of values.
        """
        if value in self.getValues():
            self._value.remove(value)
            self._valueChanged = TRUE
        else:
            raise NO_SUCH_VALUE
                    
    def displayOptionalOutput( self, canWrite, message=None ):
        """
            Return the HTML representation of the optional output
            that may be associated with the display of this
            attribute.  Output may include form buttons or
            explanitory messages.
        """
        display = ""

        if self.getSyntaxName() == 'Password' and canWrite:
            display = """
      Enter your new password and confirm it.  If you change your own
      Password, you will be logged out and returned to the search
      screen, where you can login again.  Use your new password."""

        elif self.getPointsTo() != '' and canWrite:
            display = """
      <input type="submit" name="selectDN_%s" value="Select %s">""" % \
                      ( self.getAttributeName(),
                        self.getAttributeName(),)

        return display

    def displayValues( self, canWrite, base, values=None ):
        """
            Return the HTML representation of the current list of
            values, providing a form input/selection method if the
            user can write this attribute.
        """
        syntax = self.getSyntaxName()
        if values is None:
            values = self.getValues()
        else:
            if type(values) == type(''):
                values = [values,]

        #   Strangely, the 'ou' attribute is not of DN format.  As a
        #   workaround, display anything with a non-trivial 'pointsTo'
        #   value as a link.  Hence the unexpected 'or':

        if syntax == 'DN' or self.getPointsTo() != '':
            display = self.displayValuesAsHiddenTag( values ) + \
                      self.displayValuesAsLinks(  values, base )

        elif syntax == 'Password':
            display = self.displayValuesAsPassword( canWrite )

        else:
            if canWrite:
                if self.hasValidValues():
                    display = self.displayValuesAsMultipleSelection(values)
                else:
                    display = self.displayValuesAsTextarea(values)
            else:
                display = self.displayValuesAsHiddenTag(values) + \
                          self.displayValuesAsHTMLLines(values)

        return display

    def displayValuesAsHiddenTag( self, values ):
        """
            Display value list in a hidden tag.
        """
        display = """
      <input type="hidden" name="%s:lines" value="%s">""" % \
                  ( self.getAttributeName(), 
                    self.displayValuesAsTextLines(values) )
        return display

    def displayValuesAsHTMLLines( self, values ):
        """
            Display values as lines of text, separated by '\n' characters.
        """
        display = ""
        if len( values ) > 0:
            display = values[0]
            if len( values ) > 1:
                for value in values[1:]:
                    display = display + '<br>\n' + value
        return display

    def displayValuesAsLinks( self, values, base ):
        """
            Display values as a list of links which start from base.
        """
        display = ""
        if len( values ) > 0:
            display = display + """<a href="%s%s">%s</a>""" % ( base, quote_plus(values[0]), values[0] )
            if len( values ) > 1:
                for item in values[1:]:
                    display = display + """<br>
      <a href="%s%s">%s</a>""" % ( base, item, item )
        return display

    def displayValuesAsList( self, values ):
        """
            Display values as a list of links which start from base.
        """
        display = str(values)
        return display

    def displayValuesAsMultipleSelection( self, values ):
        """
            Display values in a multiple selection input tag, listing
            valid values where the actual values are selected.
        """
        display = """
      <select name="%s:list" multiple size="%s">%s
      </select>""" % ( self.getAttributeName(),
                       str( min( len(self.getValidValues())+1, 3) ),
                       self.displayValuesAsOptionList(values) )
        return display

    def displayValuesAsOptionList( self, values, selectedTag='SELECTED' ):
        """
            Display values in an option list, listing valid values,
            and tagging actual values with the given tag.
        """
        display = ""
        for item in self.getValidValues():
            if item in values:
                display = display + """
        <option %s>%s</option>""" % ( selectedTag, item )
            else:
                display = display + """
        <option>%s</option>""" % ( item )
        return display

    def displayValuesAsPassword( self, canWrite ):
        """
            Display values as a password.  The actual value is never
            displayed, however.
        """
        #   If canWrite, then two password entry boxes are shown.
        #   Otherwise, nothing is shown.
        if canWrite:
            display = """
      <input type="password" name="%s1" size="35" value="Password"><br>
      <input type="password" name="%s2" size="35" value="Password">""" \
                       % ( self.getAttributeName(), self.getAttributeName() )
        else:
            display = ""
        return display

    def displayValuesAsTextarea( self, values ):
        """
            Display values in a textarea input box, one value per line.
        """
        display = """<textarea name="%s:lines" rows="%s" cols="35">%s</textarea>""" \
               % ( self.getAttributeName(), 
                   str( min( len(values)+1, 3) ),
                   self.displayValuesAsTextLines(values) )
        return display

    def displayValuesAsTextLines( self, values ):
        """
            Display values as lines of text, separated by '\n' characters.
        """
        display = ""
        if len( values ) > 0:
            display = values[0]
            if len( values ) > 1:
                for value in values[1:]:
                    display = display + '\n' + value
        return display

    def getValues(self):
        """
            Returns a list of values currently assigned to this
            attribute.
        """
        return getattr(self, '_value', [])

    def setValues(self, values):
        """
            Assigns a new list of values to this attribute.  Password
            syntax values are encoded.
        """
        assert type(values) == type([])

        # Empty strings are not syntactically valid values (at least
        # not in OpenLDAP).  Remove them.

        values = [item for item in values if item != '']
        
        if self.getSyntaxName() == 'Password':
            # Keep only the first value, SHA digest it, and
            # BASE64 encode that.
            newpassword = "{SHA}%s" % \
                ( base64.encodestring(sha.new( values[0]).digest()) )
            values = [newpassword,]

        if not listsAreEquivalent(self.getValues(), values):
            self._value = listCopy(values)
            self._valueChanged = TRUE

    def ValueChanged(self):
        """
            Returns TRUE if the value of this attribute has
            changed since it was last saved.
        """
        return self._valueChanged 
                
        
