"""

    ManagedSchema.py
    $Author: payne $
    $Date: 2002/04/18 19:18:42 $

    This class encapsulates the LDAP schema for a particular objectCategory
    ('person', 'group', 'orgUnit').  A singleton for each objectCategory is
    created in the STLDAPManager __init__ method.

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

__version__ = '$Revision: 1.17 $'[11:-2]

from LDAPExceptions import *
from ManagedAttributeSchema import ManagedAttributeSchema
from SchemaDefaults import *
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists, TRUE, FALSE

class ManagedSchema:

    def __init__(self,
                 defaultDictionary,
                 defaultDisplayCategories,
                 defaultNamingAttribute,
                 defaultControllers):

        #   Now, I assume that if this class instance has been persisted
        #   by the Zope persistence machinery then when it is re-inflated,
        #   the _attributeCollection will already exist.  If it doesn't,
        #   we have to create it from the appropriate default dictionary.
            
        self._pointsToTargets = defaultPointsToTargets
        
        self._DisplayCategories = listCopy(defaultDisplayCategories)
        self._Controllers = listCopy(defaultControllers)

        self._attributeCollection = {}
        self._allowedLDAPAttrNames = defaultDictionary.keys()
        self._namingAttribute = defaultNamingAttribute
        
        for ldapAttributeName in self._allowedLDAPAttrNames:
            attributeSchema = ManagedAttributeSchema(ldapAttributeName,
                                          defaultDictionary[ldapAttributeName])
            attributeSchema.setManaged(TRUE)
            self._attributeCollection[ldapAttributeName] = attributeSchema

        #   Now, because OpenLDAP allows for attributes to be subclassed 
        #   from other attributes, the attribute collection may contain 
        #   ManagedAttributeSchema objects that have a specified superior 
        #   but no other attributes.  Let's fix that now by finding 
        #   attributes with a superior, finding the superior attribute
        #   schema object, and passing it back to the target attribute's
        #   inheritsFrom method.

        for ldapAttributeName in self._allowedLDAPAttrNames:
            attributeSchema = self._attributeCollection[ldapAttributeName]
            if attributeSchema.getSup():
                superiorAttrName = attributeSchema.getSup()
                superiorAttr = self._attributeCollection[superiorAttrName]
                attributeSchema.inheritsFrom( superiorAttr )

    def addController(self, role):
        """
            Adds a role to the list of roles that are allowed to
            create and delete LDAP objects of this schema.
        """
        if not role in self._Controllers:
            self._Controllers.append(role)

    def addDisplayCategory( self, category ):
        """
            Add category to the list of display categories.
        """
        if not category in self._DisplayCategories:
            self._DisplayCategories.append( category )

    def delController(self, role):
        """
            Deletes a role from the list of roles that are allowed to
            create and delete LDAP objects of this schema.
        """
        if role in self._Controllers:
            self._Controllers.remove(role)

    def delDisplayCategory( self, category ):
        """
            Remove category from the list of display categories.
        """
        assert category in self._DisplayCategories
        self._DisplayCategories.remove( category )

    def getAllowedAttributeNames(self):
        """
            Returns a list of the names of all allowed attributes.
        """
        return listCopy( self._allowedLDAPAttrNames )

    def getAttributeCollection(self):
        """
            Returns a copy of the entire dictionary of
            ManagedAttributeSchema objects.
        """
        return self._attributeCollection.copy()

    def getAttributeSchema(self, ldapName):
        """
            Returns a named ManagedAttributeSchema object.
        """
        assert ldapName in self._allowedLDAPAttrNames
        return self._attributeCollection[ldapName]

    def getAvailableAttributeNames(self):
        """
            Returns a list of the names of all allowed attributes which
            are not managed.
        """
        available = [x for x in self._allowedLDAPAttrNames
                     if not self._attributeCollection[x].getManaged()]
        available.sort()
        return available

    def getControllers(self):
        """
            Returns a list of roles that are allowed to create and
            delete LDAP objects that follow this schema.
        """
        return self._Controllers

    def getDisplayCategories(self):
        """
            Returns the list of display categories.
        """        
        return listCopy( self._DisplayCategories )

    def getManagedAttributeNames(self):
        """
            Returns a list of the names of all managed attributes.
        """
        managed = [x for x in self._allowedLDAPAttrNames
                   if self._attributeCollection[x].getManaged()]
        return managed

    def getManagedNotRequiredAttributeNames(self):
        """
            Returns a list of the names of all managed attributes which
            are not required.
        """
        managedNR = [x for x in self.getManagedAttributeNames()
                     if not self._attributeCollection[x].getRequired()]
        managedNR.sort()
        return managedNR

    def getNamingAttribute(self):
        """
            Returns the naming attribute for this object schema.
        """
        return self._namingAttribute

    def getPointsToTargets(self):
        """
            Returns a list of pointsTo target values.
        """
        return self._pointsToTargets

    def getRequiredAttributeNames(self):
        """
            Returns a list of the names of all required attributes for this
            object.
        """
        required = [x for x in self._allowedLDAPAttrNames
                    if self._attributeCollection[x].getRequired()]
        required.sort()
        return required

    def manageAttributeSchema( self, ldapName ):
        """
            Add the named ManagedAttributeSchema object to the
            list of managed attributes.
        """
        assert self._attributeCollection.has_key(ldapName)
        self._attributeCollection[ldapName].setManaged( TRUE )

    def setAllowedAttributeNames( self, allowedNames ):
        """
            Set the (sorted) list of allowed attribute names.
        """
        allowedNames.sort()
        self._allowedLDAPAttrNames = allowedNames

    def setDisplayCategories( self, categories ):
        """
            Set display categories.
        """
        if type( categories ) == type(''):
            categories = [categories,]
        assert type( categories ) == type([])
        self._DisplayCategories[0:len(self._DisplayCategories)] = categories
        
    def setNamingAttribute( self, attributeName ):
        """
            Set the naming attribute for this object schema.
        """
        assert type(attributeName) == type('')
        if not attributeName in self._allowedLDAPAttrNames:
            raise NO_SUCH_ATTRIBUTE
        self._namingAttribute = attributeName

    def unmanageAttributeSchema( self, ldapName ):
        """
            Remove the named ManagedAttributeSchema object from the
            list of managed attributes.
        """
        assert self._attributeCollection.has_key(ldapName)

        if self._attributeCollection[ldapName].getRequired():
            raise REQUIRED_ATTRIBUTE

        self._attributeCollection[ldapName].setManaged( FALSE )
