"""

    STLDAPManager.py
    $Author: payne $
    $Date: 2002/04/10 13:52:38 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

ST LDAP Manager class.

A class that lets users authenticate against an LDAP directory, and to
search, browse, and modify the directory to an extent that depends on
their roles.

There are a number of permissions associated with this object:

    View        This default permission, granted to the Anonymous User,
                 is required for viewing public information in the
                 directory.

    Use LDAP    This permission is granted to a user who authenticates
                 against the directory.  It allows the user to change
                 certain items in his own record, and to view
                 information that is internal.

    Administer Users
                This permission, intended for Divisional Administrators
                 and system administrators, allows the user to edit,
                 add, move/rename, and delete records for people other 
                 than themselves.  It also allows them to edit, add,
                 move/rename, and delete organizational units and
                 groups.

    Manager	This permission, reserved for Zope managers, is
                 required to configure STLDAPManager instances.

"""
__version__ = '$Revision: 1.3 $'[11:-2]

from formConfigurationManager import formConfigurationManager
from formDisplayManager import formDisplayManager
from formObjectManager import formObjectManager
from formSchemaManager import formSchemaManager
from LDAPDirectory import LDAPDirectory
from ZSchema import ZSchema
from ZUtilities import ZUtilities

from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import DTMLFile, InitializeClass
from OFS.SimpleItem import SimpleItem

class STLDAPManager( formConfigurationManager,
                      formDisplayManager,
                      formObjectManager,
                      formSchemaManager,
                      ZUtilities,
                      SimpleItem ):
    """
    A persistent object for managing objects in an LDAP directory.  
    Anonymous users may search, and authenticated users may update 
    the directory, limited by their roles.
    """

    meta_type = "LDAP Directory Manager"

    manage_options = (
        formConfigurationManager.manage_options +
        formDisplayManager.manage_options +
        formObjectManager.manage_options +
        formSchemaManager.manage_options 
        )

    def __init__( self, id, title, ldapServer, directoryBase, personSearchBase, 
                  orgUnitBrowseBase, groupBrowseBase, zopeRoleSearchBase, 
                  zopeRoleSearchScope, REQUEST=None ):

        self.id = id
        self.title = title
        self.LDAPDirectory = \
            LDAPDirectory( ldapServer, directoryBase, personSearchBase, 
                           orgUnitBrowseBase, groupBrowseBase, 
                           zopeRoleSearchBase, zopeRoleSearchScope )
        user = self._getUser( self.LDAPDirectory, REQUEST )

        ZSchema( user, {'objectCategory':'person'} ).updateSchemaFromLDAP(user)
        ZSchema( user, {'objectCategory':'orgUnit'} ).updateSchemaFromLDAP(user)
        ZSchema( user, {'objectCategory':'group'} ).updateSchemaFromLDAP(user)

        self._assignRolesToPermission( 'LDAP Manage People',
                                       self.LDAPDirectory.getPersonSchema() )
        self._assignRolesToPermission( 'LDAP Manage Org Units',
                                       self.LDAPDirectory.getOrgUnitSchema() )
        self._assignRolesToPermission( 'LDAP Manage Groups',
                                       self.LDAPDirectory.getGroupSchema() )

addForm =  DTMLFile("dtml/addForm", globals())
                
def addFunction( dispatcher, id, title, ldapServer, directoryBase, 
                 personSearchBase, orgUnitBrowseBase, groupBrowseBase,
                 zopeRoleSearchBase, zopeRoleSearchScope, REQUEST=None):
    """ 
        Create a new STLDAPManager and add it to myself.
    """
    # Trap for an empty id.  Redraw the form with an error message.
    #
    if str(id) == '' and REQUEST is not None:
        message = "You must supply an id for this object."
        return dispatcher.Control_Panel.Products.STLDAPManager.addForm( 
                        dispatcher, REQUEST, manage_tabs_message=message )

    if str(ldapServer) == '' and REQUEST is not None:
        message = """You must supply an LDAP Server.  Zope must be able
                     to connect to the LDAP server to properly initialize
                     this object."""
        return dispatcher.Control_Panel.Products.STLDAPManager.addForm( 
                        dispatcher, REQUEST, manage_tabs_message=message )
    
    inst = STLDAPManager( id, title, ldapServer, directoryBase, 
                           personSearchBase, orgUnitBrowseBase, 
                           groupBrowseBase, zopeRoleSearchBase, 
                           zopeRoleSearchScope, REQUEST )
    dispatcher.Destination()._setObject(id, inst)

    if REQUEST is not None:
        dispatcher.manage_main(dispatcher, REQUEST)

InitializeClass(STLDAPManager)
