"""

    SchemaDefaults.py
    $Author: payne $
    $Date: 2002/02/26 01:10:00 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.
    
    Define default values for the user-configurable aspects of the
    schema.  These defaults are applied only when instantiating a new
    STLDAPManager object.   A few invariant items are also defined
    here.

    Almost everything here either can be changed under the Schema tab
    or should not be changed.  Changing things here can cause unit
    tests to fail.
"""

__version__ = '$Revision: 1.1 $'[11:-2]

import os, sys, re
from STLDAPUtilities import TRUE, FALSE

module = os.path.split(sys.argv[0])[1]
test = re.compile("^test.*[.]py$", re.IGNORECASE)
if test.search(module):
    testEnvironment = TRUE
else:
    testEnvironment = FALSE
    
#   These defaults include some very large dictionaries.  They have
#   been extracted into this module simply to improve the readability
#   of the rest of the code.  

#   The STLDAPManager provides a range of access to an LDAP directory,
#   depending on who is using it.  Users are categorized by their
#   Zope roles, and by a few invariant 'Internal" roles, identified by
#   the strings listed here:

defaultCanonicalRoleNames = [ 'SELF', 'ANON', 'AUTH', 'MNGR', 'IFALLOWED' ]
defaultPermanentRoles = {

        'SELF' : {
                'roleName'     : 'Self',
                'description'  : """
                                 The role controlling access to the
                                 authenticated user's own record.
                                 """ },

        'ANON' : {
                'roleName'     : 'Anonymous User',
                'description'  : """
                                 The role controlling access by an
                                 anonymous (i.e., un-authenticated) user.
                                 """ },

        'AUTH' : {
                'roleName'     : 'Authenticated User',
                'description'  : """
                                 The role controlling an authenticated
                                 user's access to the objects in the
                                 directory, in general.
                                 """ },

        'MNGR' : {
                'roleName'     : 'Direct Manager',
                'description'  : """
                                 The role controlling access to a user's
                                 record by that user's direct manager.
                                 (Not the Zope manager role.)
                                 """ },

        'IFALLOWED' : {
                'roleName'     : 'Allow access to home info',
                'description'  : """
                                 The role controlling access to the
                                 attributes each user can make
                                 available from their own record.
                                 """ },
        }

#   One more role is defined here, as a convenience, and used below.
#   Configured roles are roles granted because the user is a member
#   of a group in the LDAP directory.  To configure default
#   configured roles, add the distinguished name of the group
#   below.  You must specify it exactly given in the directory,
#   paying attention to case and spaces.

#   The System role is setup here as a role with write access to all
#   of the attributes contained in the default schema.  

DIVA = 'Divisional Administrators'
SYS  = 'System Administrators'

defaultConfiguredRoleNames = [ DIVA, SYS ]

#   Display categories provide a way of grouping the attributes for each
#   of the Group, OrgUnit, and User objects.  They are used, for example,
#   to allow attribute editing to be split across several screens.  Each
#   type of object needs at least one display category, the first of which
#   will be the default in the user interface.

defaultGroupDisplayCategories =   ['General', 'System']
defaultOrgUnitDisplayCategories = ['General', 'Telephone',
                                   'Postal Mail', 'E-Mail',
                                   'Organization', 'System']
defaultUserDisplayCategories =    ['General', 'Personal',
                                   'Telephone', 'Postal Mail',
                                   'E-Mail', 'Organization',
                                   'System']

#   For each of the Group, OrgUnit, and User types of objects, define
#   the default naming attribute.  The naming attribute is the attribute
#   that is used in constructing the relative distinguished name of the
#   LDAP object.

defaultGroupNamingAttribute = 'cn'
defaultOrgUnitNamingAttribute = 'ou'
defaultUserNamingAttribute = 'uid'

#   For each of the Group, OrgUnit, and User types of objects,
#   define which roles can, by default, create new objects
#   and delete existing objects.

defaultGroupControllers = [SYS,]
defaultOrgUnitControllers = [DIVA, SYS, ]
defaultUserControllers = [SYS,]

#   For each of the Group, OrgUnit, and User types of objects, define
#   a dictionary of LDAP attribute dictionaries.  The overal dictionary
#   is keyed on the name of the attribute.  Each attribute dictionary
#   contains as many of these keys as appropriate:
#
#     friendlyName - If given, the user interface will refer 
#           to the attribute by this name instead of by the LDAP schema
#           name.  Since LDAP attribute names are generally cryptic,
#           you probably want to give a value here for almost all
#           attributes.
#
#     readers - A list of roles that have permission to 
#           read this attribute.  Each role listed must be defined in
#           the "User Roles" section above.  In principle, this can
#           be an empty list, in which case, no one would be able to
#           read the attribute.
#
#     writers - A list of roles that have permission to 
#           modify this attribute. If not given or if an empty list
#           is given, the attribute will not be modifiable by anyone.
#
#     displayCategories - A list of names for the screens that will
#           be used to display/edit the attributes of an object.  An
#           attribute will be shown on each screen listed in its
#           displayCategories property.
#
#     validValues - A list of valid values.  If the list is not
#           empty (or ['',]) then the attribute will be treated
#           as a controlled vocabulary item.
#
#           In the case of the objectClass attribute, the valid
#           values are object classes known in the LDAP directory
#           schema.
#
#     requiredValues - A list of required values.  If the list is not
#           empty (or ['',]) then the attribute will be treated
#           as a controlled vocabulary item.
#
#           In the case of the objectClass attribute, the required
#           values make up the list of object classes that will be
#           specified when a new object is created in the LDAP
#           directory.  It is important that this list be complete
#           in the sense that all of the superclasses for all of
#           the items in the list are also in the list.  For example
#
#               ['top','person','organizationalPerson','inetOrgPerson']
#
#           is complete, but
#
#               ['top','person','inetOrgPerson']
#
#           is not because 'organizationalPerson' is the superclass
#           for 'inetOrgPerson'.
#
#     pointsTo - For attributes whose value is a distinguished name,
#           the type of object that the distinguished name points to.

defaultPointsToTargets = [ '', 'person', 'orgUnit', 'group' ]

defaultGroupSchema = {

        'businessCategory' : {
                'friendlyName'      : 'Category',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['General', 'System'], },

        'cn' : {
                'friendlyName'      : 'Group Name',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['General', 'System'],
                'required'          : TRUE, },

        'description' : {
                'friendlyName'      : 'Description',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['General', 'System'], },

        'o' : {
                'friendlyName'      : 'Organization',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['System',], },

        'objectClass' : {
                'friendlyName'      : 'Object Class',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['System',],
                'validValues'       : ['groupOfUniqueNames', 'top'],
                'required'          : TRUE,
                'requiredValues'    : ['groupOfUniqueNames', 'top'], },

        'ou' : {
                'friendlyName'      : 'Organizational Unit',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['System',], },

        'owner' : {
                'friendlyName'      : 'Owner',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['System',],
                'pointsTo'          : 'person', },

        'seeAlso' : {
                'friendlyName'      : 'See Also',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['System', 'General'],
                'pointsTo'          : 'person', },

        'uniqueMember' : {
                'friendlyName'      : 'Members',
                'readers'           : ['SELF', 'AUTH', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['System', 'General'],
                'pointsTo'          : 'person' },
    }

#   defautlOrgUnitSchema defines the schema for the LDAP 'organizational
#   unit' object.  At STScI, we have defined our own org unit schema by
#   subclassing of the LDAP ou class.  To revert to using the standard
#   LDAP ou class, simply remove all attributes that have the 'stsci'
#   prefix and remove 'STScIOrganizationalUnit' from the validValues
#   and requiredValues lists for the objectClass attribute.

defaultOrgUnitSchema = {

        'businessCategory' : {
                'friendlyName'      : 'Category',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',], },

        'description' : {
                'friendlyName'      : 'Description',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',], },

        'facsimileTelephoneNumber' : {
                'friendlyName'      : 'FAX Number',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Telephone',], },

        'l' : {
                'friendlyName'      : 'Location',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'manager' : {
                'friendlyName'      : 'Manager',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',],
                'pointsTo'          : 'person', },

        'objectClass' : {
                'friendlyName'      : 'Object Class',
                'readers'           : ['AUTH', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['System',],
                'validValues'       : ['stsciOrganizationalUnit',
                                       'organizationalUnit', 'top'],
                'requiredValues'    : ['stsciOrganizationalUnit',
                                       'organizationalUnit', 'top'],
                'required'          : TRUE, },

        'ou' : {
                'friendlyName'      : 'Organizational Unit Name',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',],
                'required'          : TRUE,  },

        'physicalDeliveryOfficeName' : {
                'friendlyName'      : 'Mailstop',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail', 'General'], },

        'postalAddress' : {
                'friendlyName'      : 'Postal Address',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'postalCode' : {
                'friendlyName'      : 'Postal Code',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'postOfficeBox' : {
                'friendlyName'      : 'Post Office Box',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'searchGuide' : {
                'friendlyName'      : 'Search Guide',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',], },

        'seeAlso' : {
                'friendlyName'      : 'Home Page',           
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',],
                'pointsTo'          : 'person', },

        'st' : {
                'friendlyName'      : 'State',          
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'street' : {
                'friendlyName'      : 'Street Address',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'stsciAdministrator' : {
                'friendlyName'      : 'Divisional Administrator',
                'readers'           : ['AUTH', 'MNGR', DIVA, DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',], },

        'telephoneNumber' : {
                'friendlyName'      : 'Telephone Number',
                'readers'           : ['ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Telephone',], }
      }

defaultUserSchema = {

        'businessCategory' : {
                'friendlyName'      : 'Category',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,SYS,],
                'displayCategories' : ['General',], },

        'carLicense' : {
                'friendlyName'      : 'Automobile License',
                'readers'           : ['SELF', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Personal',], },

        'cn' : {
                'friendlyName'      : 'Common Name(s)',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['General','Personal'], 
                'required'          : TRUE, },

        'departmentNumber'  : {
                'friendlyName'      : 'Branch',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, SYS,],
                'displayCategories' : ['General',], },

        'description' : {
                'friendlyName'      : 'Description',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['General',], },

        'displayName' : {
                'friendlyName'      : 'Display Name',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['General',], },

        'employeeType' : {
                'friendlyName'      : 'Employee Type',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',],
                'validValues'       : ['Full-Time',  'Part-Time',
                                       'Intern',     'Contractor',
                                       'Consultant', 'Inactive'] },

        'facsimileTelephoneNumber' : {
                'friendlyName'      : 'FAX Number',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, 'SELF'],
                'displayCategories' : ['Telephone',], },

        'givenName' : {
                'friendlyName'      : 'Given Name',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['Personal',], },

        'homePostalAddress' : {
                'friendlyName'      : 'Home Address',
                'readers'           : ['SELF', 'MNGR', DIVA, 'IFALLOWED'],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Personal',], },

        'homePhone'  : {
                'friendlyName'      : 'Home Phone',
                'readers'           : ['SELF', 'MNGR', DIVA, 'IFALLOWED'],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Personal',], },

        'initials' : {
                'friendlyName'      : 'Initials',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['Personal',], },

        'l' : {
                'friendlyName'      : 'Location',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['Postal Mail',], },

        'mail' : {
                'friendlyName'      : 'Email Address',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['E-Mail', 'General'], },

        'mailAlternateAddress' : {
                'friendlyName'      : 'Mail Published Address',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['E-Mail',], },

        'mailHost' : {
                'friendlyName'      : 'Mail Host',
                'readers'           : [SYS,],
                'writers'           : [SYS,],
                'displayCategories' : ['E-Mail',], },

        'mailRoutingAddress' : {
                'friendlyName'      : 'Mail Routing Address',
                'readers'           : [SYS,],
                'writers'           : [SYS,],
                'displayCategories' : ['E-Mail',], },

        'manager' : {
                'friendlyName'      : 'Manager',
                'readers'           : ['SELF', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',],
                'pointsTo'          : 'person'  },

        'mobile' : {
                'friendlyName'      : 'Cell Phone',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Telephone',], },

        'o': {
                'friendlyName'      : 'Organization',
                'readers'           : ['SELF', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, SYS,],
                'displayCategories' : ['Organization',], },

        'objectClass' : {
                'friendlyName'      : 'Object Class',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['System',],
                'validValues'       : ['stsciPerson',
                                       'inetOrgPerson',
                                       'organizationalPerson',
                                       'person',
                                       'top',
                                       'mailRecipient'],
                'requiredValues'    : ['stsciPerson',
                                       'inetOrgPerson',
                                       'organizationalPerson',
                                       'person',
                                       'top'], 
                'required'          : TRUE, },

        'ou' : {
                'friendlyName'      : 'Organizational Unit',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['Organization',],
                'pointsTo'          : 'orgUnit',  },

        'pager' : {
                'friendlyName'      : 'Pager',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Telephone',], },

        'physicalDeliveryOfficeName' : {
                'friendlyName'      : 'Mail Stop',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'postalAddress' : {
                'friendlyName'      : 'Postal Address',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, SYS,],
                'displayCategories' : ['Postal Mail',], },

        'postalCode' : {
                'friendlyName'      : 'Postal Code',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, SYS,],
                'displayCategories' : ['Postal Mail',], },

        'postOfficeBox' : {
                'friendlyName'      : 'Post Office Box',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Postal Mail',], },

        'roomNumber' : {
                'friendlyName'      : 'Office Number',
                'readers'           : ['SELF', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA, SYS],
                'displayCategories' : ['General',], },

        'secretary' : {
                'friendlyName'      : 'Secretary',
                'readers'           : ['SELF', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',],
                'pointsTo'          : 'person' },

        'seeAlso' : {
                'friendlyName'      : 'Home Page',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['General',], 
                'pointsTo'          : 'person' },

        'sn' : {
                'friendlyName'      : 'Surname',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Personal',], 
                'required'          : TRUE, },

        'st' : {
                'friendlyName'      : 'State',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'street' : {
                'friendlyName'      : 'Street Address',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['Postal Mail',], },

        'stsciAccessCardNumber' : {
                'friendlyName'      : 'Access Card Number(s)',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : [DIVA,],
                'displayCategories' : ['Organization',], },

        'stsciContactAddress' : {
                'friendlyName'      : 'Contact Address',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Organization',], },

        'stsciContactName' : {
                'friendlyName'      : 'Contact Name',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Organization',], },

        'stsciContactPhone' : {
                'friendlyName'      : 'Contact Phone',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Organization',], },

        'stsciKeyCodes': {
                'friendlyName'      : 'Key Codes',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Postal Mail',], },

        'stsciParkingSticker' : {
                'friendlyName'      : 'Parking Sticker Number',
                'readers'           : ['SELF', 'MNGR', DIVA],
                'writers'           : ['SELF', DIVA],
                'displayCategories' : ['Organization',], },

        'telephoneNumber' : {
                'friendlyName'      : 'Telephone Number',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA, SYS,],
                'displayCategories' : ['Telephone',], },

        'title' : {
                'friendlyName'      : 'Title',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [DIVA,],
                'displayCategories' : ['General',], },

        'userPassword' : {
                'friendlyName'      : 'User Password',
                'readers'           : ['SELF', SYS],
                'writers'           : ['SELF', SYS],
                'displayCategories' : ['General',], },

        'uid' : {
                'friendlyName'      : 'Username',
                'readers'           : ['SELF', 'ANON', 'AUTH', 'MNGR', DIVA, SYS],
                'writers'           : [SYS,],
                'displayCategories' : ['System',], },
    }
