"""
    ZObject.py
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 19:18:42 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    The ZObject class is a Zope-aware object that instantiates
    an internal LDAP object appropriate for the DN information
    passed in the REQUEST.  It contains methods used for drawing
    the forms to display and LDAP object's attributes, and for
    updating the LDAP object from REQUEST information submitted
    from forms.
"""

__version__ = '$Revision: 1.19 $'[11:-2]

from AccessControl import getSecurityManager
from DocumentTemplate.DocumentTemplate import HTML
from urllib import quote_plus, unquote_plus, urlencode

from LDAPExceptions import *
from LDAPObject import LDAPObject
from PeopleList import PeopleList
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists, TRUE, FALSE

class ZObject:

    def __init__( self, user, REQUEST ):

        #   Get an LDAPObject.  

        self._object = LDAPObject( user )
        distinguishedName = REQUEST.get( 'distinguishedName', None )

        self._IChangedMyOwnPassword = FALSE

        if distinguishedName is not None:
            distinguishedName = unquote_plus( distinguishedName )
            self._object.refresh( distinguishedName )
            objectCategory = self._object.getObjectCategory()
        elif REQUEST.has_key( 'objectCategory' ):
            objectCategory = REQUEST['objectCategory']
        else:
            objectCategory = 'None'

        REQUEST.set( 'objectCategory', objectCategory )
        REQUEST.set( 'ZObject', self )

    def canControl( self, objectCategory ):
        """
            Can the user be a controller for objects of the specified
            objectCategory?  That is, does the user have any roles
            in the ManagedSchema's controllers list.
        """
        if objectCategory.lower() == 'person':
            controllers = \
             self._object.user.LDAPDirectory.getPersonSchema().getControllers()
        elif objectCategory.lower() == 'orgunit':
            controllers = \
             self._object.user.LDAPDirectory.getOrgUnitSchema().getControllers()
        if objectCategory.lower() == 'group':
            controllers = \
             self._object.user.LDAPDirectory.getGroupSchema().getControllers()

        for item in getSecurityManager().getUser().getRoles():
            if item in controllers:
                return TRUE
        return FALSE

    def getGroupMembers( self ):
        """
            If the saved LDAPObject has been refreshed with information
            from a groupOfNames or groupOfUniqueNames in the directory,
            then return the list of values (DNs) of the member or
            uniqueMember attribute, respectively.
        """
        if self._object.getObjectCategory() == 'group':
            if 'member' in self._object._ManagedAttributes.keys():
                return self._object._ManagedAttributes['member'].getValues()
            elif 'uniqueMember' in self._object._ManagedAttributes.keys():
                return self._object._ManagedAttributes['uniqueMember'].getValues()
            else:
                return []

        else:
            return []

    def getObject( self ):
        """
            Returns the saved LDAPObject object
        """
        return self._object

    def getObjectClasses( self ):
        """
            Returns the values of the objectClass attribute for the
            saved LDAPObject
        """
        return self._objectClasses

    def iChangedMyOwnPassword( self ):
        """
            Returns the value of the attribute that checkes whether 
            I changed my own password.
        """
        return self._IChangedMyOwnPassword

    def selectDNKey( self, REQUEST ):
        """
            If the REQUEST contains a key of the form 'selectDN_ldapName'
            this method returns 'ldapName'
        """
        for key in REQUEST.keys():
            if key[0:9] == 'selectDN_':
                return key[9:]
        return ''

    def updateDirectoryFromRequest( self, REQUEST ):
        """
            Make directory updates according to the REQUEST variables.
        """

        #   Invoke a method that matches the name of the submit
        #   button (with '_' prepended).

        if REQUEST.has_key( 'updateObjectValues' ):
            message = self._updateObjectValues( REQUEST )

        elif self.selectDNKey( REQUEST ) != '':
            message = self._selectDN( REQUEST )

        elif REQUEST.has_key( 'login' ):
            message = self._login( REQUEST )

        elif REQUEST.has_key( 'addObject' ):
            message = self._addObject( REQUEST )

        elif REQUEST.has_key( 'delObject' ):
            message = self._delObject( REQUEST )

        elif REQUEST.has_key( 'addMembers' ):
            message = self._addMembers( REQUEST )

        elif REQUEST.has_key( 'removeMembers' ):
            message = self._removeMembers( REQUEST )

        else:
            # You can get here if your form has a single text input
            # field and you press return rather than the form button.
            # Maybe there is a way to figure it out, maybe we don't
            # care.

            message = ""

        return message

    def _addObject( self, REQUEST ):

        #   Combine items from the REQUEST to form the distinguished
        #   name and object category of the object to be added.  Add
        #   and redirect to a display of it.

        namingAttribute = REQUEST.get( 'namingAttribute' )
        namingAttributeValue = REQUEST.get( 'namingAttributeValue' )
        parentDN = unquote_plus( REQUEST.get( 'parentDN' ) )
        objectCategory = REQUEST.get( 'objectCategory' )

        distinguishedName = "%s=%s,%s" % ( namingAttribute,
                                           namingAttributeValue,
                                           parentDN )

        self._object.create( distinguishedName, objectCategory )

        message = "Added object '%s'.  " % ( distinguishedName )

        url = "%s?distinguishedName=%s&manage_tabs_message=%s" % \
                  ( REQUEST['URL1'], 
                    quote_plus(distinguishedName), 
                    quote_plus(message) )
        REQUEST.RESPONSE.redirect( url )

    def _delObject( self, REQUEST ):
        if not REQUEST.has_key('areYouSure'):
            REQUEST.set( 'areYouSure', '1' )
            return "ARE YOU SURE?"

        elif REQUEST.has_key('ids'):
            ids = REQUEST.get( 'ids', [] )
            for distinguishedName in ids:
                self._object.refresh( distinguishedName )
                self._object.delete( REQUEST.get('areYouSure') )
        else:
            targetDN = REQUEST.get('targetDN') 
            self._object.refresh( targetDN )
            self._object.delete( REQUEST.get('areYouSure') )

        message = "Objects deleted."

        url = "%s?objectFormID=delObject" % ( REQUEST['URL0'] )
        REQUEST.RESPONSE.redirect( url )

    def _addMembers( self, REQUEST ):
        ids = REQUEST.get( 'ids', [] )
        for distinguishedName in ids:
            if 'member' in self._object._ManagedAttributes.keys():
                memberAttr = self._object.getManagedAttribute('member')
            elif 'uniqueMember' in self._object._ManagedAttributes.keys():
                memberAttr = self._object.getManagedAttribute('uniqueMember')
            else:
                memberAttr = None

            if memberAttr is not None:
                memberAttr.addValue( distinguishedName )

            if len( ids ) > 0:
                self._object.updateDirectory()
                message = "Added group members"
            else:
                message = ""

        return message

    def _removeMembers( self, REQUEST ):
        if not REQUEST.get('areYouSure',0):
            REQUEST.set( 'areYouSure', 1 )
            return "ARE YOU SURE?"

        elif REQUEST.has_key('ids'):
            REQUEST.set('areYouSure',0)
            ids = REQUEST.get( 'ids', [] )
            for distinguishedName in ids:
                if 'member' in self._object._ManagedAttributes.keys():
                    memberAttr = self._object.getManagedAttribute('member')
                elif 'uniqueMember' in self._object._ManagedAttributes.keys():
                    memberAttr = self._object.getManagedAttribute('uniqueMember')
                else:
                    memberAttr = None

                if memberAttr is not None:
                    memberAttr.delValue( distinguishedName )

            if len( ids ) > 0:
                self._object.updateDirectory()
                message = "Removed group members"
            else:
                message = ""

        return message

    def _login( self, REQUEST ):
        if not 'AUTH' in self._object.user.getRoles():
            REQUEST.RESPONSE.setStatus(401)
            REQUEST.RESPONSE.setHeader('WWW-Authenticate','basic realm=')
        else:
            REQUEST.RESPONSE.redirect( REQUEST.get( 'URL1' ) )

    def _selectDN( self, REQUEST ):

        targetAttribute = self.selectDNKey( REQUEST )

        #   If we got here from the _selectDNForm then update the REQUEST.
        #   Furthermore, refresh my _object with the selected DN, if
        #   we were selecting an OrgUnit or group, which are selected
        #   singly.

        if REQUEST.has_key('newDN'):
            option = REQUEST.get( 'selectDN_' + targetAttribute )
            objectCategory = REQUEST.get( 'objectCategory' )

            if option == " Select ":
                REQUEST.set( targetAttribute, REQUEST.get( 'ids' ) )
            elif option == " Nobody ":
                REQUEST.set( targetAttribute, [] )
            elif objectCategory in ['orgUnit','group']:
                targetDN = REQUEST.get(targetAttribute)
                self._object.refresh( targetDN )

            del REQUEST.form['newDN']
            del REQUEST.form['targetAttribute']
            REQUEST.set( 'objectFormID', 'display' )
            return "Selected %s.  You still need to Save." % ( targetAttribute )

        #   Otherwise, set REQUEST variables so that one of the
        #   _select(objectCategory)DNForm gets
        #   displayed next.

        else:
            REQUEST.set( 'targetAttribute', targetAttribute )

            if REQUEST.has_key( 'pointsTo' ):
                pointsTo = REQUEST['pointsTo']

            elif hasattr( self._object, 'managedSchema' ):
                attrSchema = self._object.managedSchema.getAttributeSchema( targetAttribute )
                pointsTo = attrSchema.getPointsTo()

            else:
                pointsTo = 'orgUnit'

            if pointsTo.lower() == 'person':
                REQUEST.set('objectFormID', 'selectPersonDN' )
            elif pointsTo.lower() == 'personorgunit':
                REQUEST.set('objectFormID', 'selectPersonOrgUnitDN' )
            elif pointsTo.lower() == 'orgunit':
                REQUEST.set('objectFormID', 'selectOrgUnitDN' )
            elif pointsTo.lower() == 'group':
                REQUEST.set('objectFormID', 'selectGroupDN' )
            return REQUEST['objectFormID']

    def _updateObjectValues( self, REQUEST ):

        #   Pull out all of the ManagedAttributes in this displayCategory
        #   that the user can write.  Update them with values from the
        #   REQUEST, but them back, and update the directory.

        distinguishedName = REQUEST.get( 'distinguishedName' )
        displayCategory = REQUEST.get( 'displayCategory' )
        object = self._object
        object.refresh()

        attrNames = object.getManagedAttributeNames( displayCategory )
        attrNames = [item for item in attrNames 
                     if object.canWriteAttribute(item)]

        for attrName in attrNames:
            attrObject = object.getManagedAttribute( attrName )

            #   Special case for paswords, which need two values in
            #   the REQUEST.

            if attrObject.getSyntaxName() == 'Password':
                pw1 = REQUEST.get( attrName+'1', '' )
                pw2 = REQUEST.get( attrName+'2', '' )

                if pw1 != pw2:
                    REQUEST.set( attrName+'1', 'Password' )
                    REQUEST.set( attrName+'2', 'Password' )
                    return "Passwords do not match.  Try again."

                elif pw1 == 'Password':
                    break

                else:
                    attrObject.setValues( [pw1,] )
                    self._IChangedMyOwnPassword = TRUE

            else:
                attrObject.setValues( REQUEST.get( attrName, [] ) )

            object.setManagedAttribute( attrObject )

        object.updateDirectory()
        return "ZObject: Updated values in the directory."
   
    def showPeopleList(self, peopleQuery, sortKey ):
        """
            Returns a list of people that match the query.

            The argument peopleQuery can be in one of two forms:

            1.  A simple string (eg. 'hanisch') in which case
                we want to return all of the folks for who have
                that value in any of the normal telphone list
                attribtes.

            2.  A fully qualified distinguishedName of an ou, in
                which case, we want to return all of the folks in
                that ou.

        """
        pList = PeopleList( self._object.user, peopleQuery )
        return pList.sortBy( sortKey )
