"""

    Module: $RCSfile: ZSchema.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 19:18:42 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    The ZSchema class is the mutable schema object class.  It holds
    an AuthenticatedUser instance, a ManagedSchema class instance,
    and a ManagedRoles instance, and contains methods used on the
    various faces of the Schema tab.

"""
__version__ = '$Revision: 1.35 $'[11:-2]

from LDAPExceptions import REQUIRED_ATTRIBUTE, NO_SUCH_ATTRIBUTE
from LDIFObjectParser import LDIFObjectParser
from LDIFAttributeParser import LDIFAttributeParser
from ManagedAttributeSchema import ManagedAttributeSchema
from ZRoles import ZRoles
from STLDAPUtilities import _ldap, DNNormalize, DNsAreEquivalent, \
    getNameFromDN, listsAreEquivalent, listCopy, list1ContainsList2, \
    mergeLists, TRUE, FALSE

class ZSchema:

    def __init__(self, user, REQUEST ):

        #   Update the Zope schema object with values from the LDAP 
        #   schema (and save the list of available LDAP object classes).  
        
        self._user = user

        if not REQUEST.has_key('objectCategory'):
            REQUEST.set('objectCategory','person')
        category = REQUEST['objectCategory'].lower()

        if category == 'group':
            self._schema = user.getLDAPDirectory().getGroupSchema()
        elif category == 'person':
            self._schema = user.getLDAPDirectory().getPersonSchema()
        elif category == 'orgunit':
            self._schema = user.getLDAPDirectory().getOrgUnitSchema()
        else:
            assert FALSE            

        self._ldapObjects = self.updateSchemaFromLDAP( user )

        #   Instantiate a mutable ZRoles object from the supplied
        #   ManagedRoles object, and save it as an attribute.

        self._managedRoles = user.getLDAPDirectory().getManagedRoles()
        
        #   Initialize this object's attribute to those of the updated
        #   Zope schema.  These assignments make changes to this
        #   object persistent.

        REQUEST['ZSchema'] = self
        
    def checkObjectClassCompleteness( self, objectClasses ):
        """
            Given a list of objectClasses, check that the list is
            complete, in the sense that all of the superclasses
            for each item in the list are also in the list.
        """
        for className in objectClasses:
            superClasses = getLDAPSuperClassNames( className,
                                                   self._ldapObjects )
            for item in superClasses:
                if item not in objectClasses:
                    return FALSE
        return TRUE

    def DNsAreEquivalent( self, dn1, dn2, REQUEST=None ):
        """
           Wrapper for STLDAPUtilities.DNsAreEquivalent
        """
        return DNsAreEquivalent( dn1, dn2 )

    def getManagedRoles(self):
        """
            Returns a managed roles object.
        """
        return self._managedRoles

    def getManagedSchema(self):
        """
            Returns a managed schema object,
        """
        return self._schema

    def getUnmanagedObjectClasses( self ):
        """
            Returns a list of names of all of the object classes in 
            the LDAP schema, except for the ones already managed 
            (i.e., either in available or selected object class names).
        """
        objectClassSchema = self._schema.getAttributeSchema( 'objectClass' )
        managed = mergeLists( objectClassSchema.getValidValues(),
                              objectClassSchema.getRequiredValues() )
        unmanaged = [item for item in self._ldapObjects.keys()
                     if item not in managed]
        unmanaged.sort()
        return unmanaged

    def getUser( self ):
        """
            Returns the AuthenticatedUser object
        """
        return self._user

    def updateSchemaFromLDAP( self, user ):
        """
            Retrieves the schema from the LDAP directory server
            and updates the persisted schema collection.
        """
        LDAPschemaDN = user.search( "", _ldap.SCOPE_BASE, "(objectclass=*)",
            ['subschemaSubentry',])[0][1]['subschemaSubentry'][0]

        LDAPschema = user.search( LDAPschemaDN, _ldap.SCOPE_BASE,
            "(objectclass=*)", ["objectClasses","attributeTypes"] )[0][1]

        ldapObjects = {}
        for ldifExpression in LDAPschema['objectClasses']:
            parsedObject = LDIFObjectParser( ldifExpression )
            ldapObjects[parsedObject.name.lower()] = parsedObject

        ldapAttributes = {}
        for ldifExpression in LDAPschema['attributeTypes']:
            parsedObject = LDIFAttributeParser( ldifExpression )
            ldapAttributes[parsedObject.name.lower()] = parsedObject              
        
        #   Get the attribute names defined in the LDAP directory
        #   schema for this object.

        requiredLDAPAttrNames = []
        optionalLDAPAttrNames = []
        
        objectClasses = \
            self._schema.getAttributeSchema('objectClass').getValidValues()

        for objectClass in objectClasses:
            
            required, optional = getLDAPAttributeNames(objectClass, ldapObjects)
            requiredLDAPAttrNames = mergeLists(requiredLDAPAttrNames, required)
            optionalLDAPAttrNames = mergeLists(optionalLDAPAttrNames, optional)
                                       
        optionalLDAPAttrNames = [item for item in optionalLDAPAttrNames \
                                 if not item in requiredLDAPAttrNames ]
                                  
        allowedLDAPAttrNames = mergeLists(requiredLDAPAttrNames,
                                          optionalLDAPAttrNames )


        allowedLDAPAttrNames.sort()
        self._schema._allowedLDAPAttrNames = allowedLDAPAttrNames

        #   Update the attribute collection items that are currently
        #   allowed.  Items no longer allowed, because the objectClass
        #   that defined has been dropped, remain untouched.

        for ldapName in allowedLDAPAttrNames:

            if ldapName in self._schema._attributeCollection.keys():
                metadata = \
                    self._schema.getAttributeSchema(ldapName).getMetadata()
            else:
                metadata = { 'managed': FALSE,}

            parsedAttribute = ldapAttributes[ldapName.lower()]
            metadata['syntax'] = parsedAttribute.syntax
            if ldapName in requiredLDAPAttrNames:
                metadata['required'] = TRUE

            managedAttribute = ManagedAttributeSchema( ldapName, metadata )
            self._schema._attributeCollection[ ldapName ] = managedAttribute

        for ldapName in self._schema._attributeCollection.keys():
            if not ldapName in allowedLDAPAttrNames:
                del self._schema._attributeCollection[ldapName]                

        #   Force the namingAttribute to be required.

        self._schema.getAttributeSchema(self._schema.getNamingAttribute()).setRequired(TRUE)

        #   The list of available LDAP objects is used to check new
        #   values for the 'objectClass' attribute, so we return it
        #   for ZSchema use.  But the ManagedSchema object doesn't 
        #   need it.

        return ldapObjects
    
    def updateSchemaFromRequest( self, REQUEST ):
        """
            Change the schema according to the REQUEST variables.
        """
        message = None

        #   Invoke a method that matches the name of the submit
        #   button (with '_' prepended).

        if REQUEST.has_key( 'saveAttributeSchema' ):
            message = self._saveAttributeSchema( REQUEST )

        elif REQUEST.has_key( 'setNamingAttribute' ):
            message = self._setNamingAttribute( REQUEST )

        elif REQUEST.has_key( 'moveAttributeToAvailable' ):
            message = self._moveAttributeToAvailable( REQUEST )

        elif REQUEST.has_key( 'moveAttributeToManaged' ):
            message = self._moveAttributeToManaged( REQUEST )

        elif REQUEST.has_key( 'moveObjectClassesToSelected' ):
            message = self._moveObjectClassesToSelected( REQUEST )

        elif REQUEST.has_key( 'moveObjectClassesToAvailable' ):
            message = self._moveObjectClassesToAvailable( REQUEST )

        elif REQUEST.has_key( 'dropObjectClasses' ):
            message = self._dropObjectClasses( REQUEST )

        elif REQUEST.has_key( 'addAvailableObjectClasses' ):
            message = self._addAvailableObjectClasses( REQUEST )

        elif REQUEST.has_key( 'changeDisplayCategories' ):
            message = self._changeDisplayCategories( REQUEST )

        elif REQUEST.has_key( 'renameDisplayCategories' ):
            message = self._renameDisplayCategories( REQUEST )

        elif REQUEST.has_key( 'saveAttrsByCategory' ):
            message = self._saveAttrsByCategory( REQUEST )

        elif REQUEST.has_key( 'saveAttrsByRole' ):
            message = self._saveAttrsByRole( REQUEST )

        elif REQUEST.has_key( 'addControllers' ):
            message = self._addControllers( REQUEST )

        elif REQUEST.has_key( 'deleteControllers' ):
            message = self._deleteControllers( REQUEST )

        elif REQUEST.has_key( 'changeRoles' ):
            zRoles = ZRoles( self._user, self._managedRoles )
            message = zRoles.updateRolesFromRequest( REQUEST )

        else:
            # You can get here if your form has a single text input
            # field and you press return rather than the form button.
            # Maybe there is a way to figure it out:

            message = "No method to perform requested action."

        return message

    def _addAvailableObjectClasses( self, REQUEST ):

        # A method for adding items to the available object class
        # list.  Each input name is checked against the LDAP
        # schema to make sure it exists.

        newClasses = REQUEST.get('newClasses',None)
        if newClasses is None:
            return

        schema = self._schema.getAttributeSchema( 'objectClass' )
        valid = schema.getValidValues()
        added = []
        bogus = []
        super = []

        for newClass in newClasses:
            if self._ldapObjects.has_key( newClass.lower() ):

                # Get canonical name.

                name = self._ldapObjects[newClass.lower()].name
                schema.addValidValues( name )
                valid.append( name )
                added.append( name )

                # Do any superclasses also need to be added?

                supers = getLDAPSuperClassNames( name,
                                                      self._ldapObjects )
                supersToAdd = [item for item in supers
                               if item not in valid]
                for superClass in supersToAdd:
                    name = self._ldapObjects[superClass.lower()].name
                    schema.addValidValues( name )
                    valid.append( name )
                    super.append( name )

            else:
                bogus.append( name )

        message = ""
        if len( added ) > 0:
            message = message + \
                      "Added object classes %s.  " % ( str( added ) )
        if len( super ) > 0:
            message = message + \
                      "Added missing superClasses %s.  " % ( str( super ) )
        if len( bogus ) > 0:
            message = message + \
                      "No matching LDAP classes for %s." % ( str( bogus ) )
        return message

    def _addControllers( self, REQUEST ):

        # A method for adding the the controllers list

        ids = REQUEST.get( 'newRoles' )

        zopeRoleSearchBase = \
            self._user.getLDAPDirectory().getZopeRoleSearchBase()
        for item in ids:
            if self._user.getMatchingGroups( item, zopeRoleSearchBase )!=[]:
                self._schema.addController( item )
        return "Added roles."

    def _changeDisplayCategories( self, REQUEST ):

        # A method for updating the display categories list

        option = REQUEST.changeDisplayCategories
        ids = REQUEST.get( 'ids' )

        if option == " Delete " and len( ids ) > 0:
            for item in ids:
                if item in self._schema.getDisplayCategories():
                    self._schema.delDisplayCategory( item )
            message = "Deleted object categories."

        elif option == " First " and len( ids ) > 0:
            displayCategories = [item for item in 
                                 self._schema.getDisplayCategories()
                                 if item in ids] + \
                                [item for item in 
                                 self._schema.getDisplayCategories()
                                 if item not in ids]
            self._schema.setDisplayCategories( displayCategories )
            message = "Re-ordered display categories."

        elif option == " Add ":
            newCategory = REQUEST.get( 'newCategory', None )
            if newCategory:
                self._schema.addDisplayCategory( newCategory )
            message = "Added object category."

        else:
            message = ""

        return message

    def _deleteControllers( self, REQUEST ):

        # A method for dropping items from the controllers list

        ids = REQUEST.get( 'ids' )
        for item in ids:
            self._schema.delController( item )
        return "Deleted roles."

    def _dropAvailableObjectClasses( self, dropClasses, REQUEST ):

        # Drop the specifed object classes from the available
        # object classes list.

        objectClassSchema = self._schema.getAttributeSchema( 'objectClass' )
        objectClassSchema.delValidValues( dropClasses )
        return "Dropped specified objectClasses."

    def _dropObjectClasses( self, REQUEST ):

        # Drop object classes from both the available and selected
        # object class lists.  

        message = ""
        availClasses = REQUEST.get('availClasses',[])
        if len( availClasses ) > 0:
            message = message + \
                      self._dropSelectedObjectClasses( availClasses, REQUEST )

        selectClasses = REQUEST.get('selectClasses',[])
        if len( selectClasses ) > 0:
            message = message + \
                      self._dropAvailableObjectClasses( selectClasses, REQUEST )

        return message

    def _dropSelectedObjectClasses( self, dropClasses, REQUEST ):

        # Drop the specified classes from the selected object classes
        # list.  But you can't drop an object class subclassed by one
        # of the classes remaining in the list.  In that case, the
        # drop fails.  Compute what the new selected object classes list 
        # would be, and see if it is complete.

        objectClassSchema = self._schema.getAttributeSchema( 'objectClass' )
        requiredValues = objectClassSchema.getRequiredValues()
        for item in dropClasses:
            requiredValues.remove( item )
        if not self.checkObjectClassCompleteness( requiredValues ):

            # What classes would be stranded with a missing superclass?

            stranded = []
            for item in requiredValues:
                supers = getLDAPSuperClassNames( item,
                                                      self._ldapObjects )
                if not list1ContainsList2( requiredValues, supers ):
                    stranded.append( item )

            message = """
                      The requested object class(es), %s, could not 
                      be moved because the list contains superclasses
                      for classes %s.  Only mixin and lowest level 
                      classes can be moved.  Either move all of these
                      classes, or pick items from the second list.
                      """ % ( str(dropClasses), str(stranded) )

            return message

        #   The REQUEST is missing the 'validValues' and 'requiredValues'
        #   keys that any other attribute schema update would have.
        #   Supply them, and then call the _saveAttributeSchema method.

        objectClassSchema.setRequiredValues( requiredValues )

        message = self._saveAttributeSchema( REQUEST )
        return "Dropped specified objectClasses.  " + str(message)

    def _moveAttributeToAvailable( self, REQUEST ):

        # A method for making an attribute "available," rather than
        # "managed."

        selectedAttrs = REQUEST.get('unmanageAttrs',[])
        for item in selectedAttrs:
            self._schema.unmanageAttributeSchema( item )

        return "Updated attribute lists."

    def _moveAttributeToManaged( self, REQUEST ):

        # A method for making an attribute "managed," rather than
        # "available."

        selectedAttrs = REQUEST.get('manageAttrs',[])
        for item in selectedAttrs:
            self._schema.manageAttributeSchema( item )

        return "Updated attribute lists."

    def _moveObjectClassesToAvailable( self, REQUEST ):

        # A method for making an object class value "available,"
        # rather than "selected."  But you can't unselect an
        # object class subclassed by some other object in the
        # required list.

        availClasses = REQUEST.get('availClasses',[])
        message = self._dropSelectedObjectClasses( availClasses, REQUEST )
        if message[0:31] == "Dropped specified objectClasses.":
            # i.e., the drop worked
            objectClassSchema = self._schema.getAttributeSchema( 'objectClass' )
            objectClassSchema.addValidValues( availClasses )
            message = self._saveAttributeSchema( REQUEST )
            return "Moved specified objectClasses.  " + str(message)

        else:
            return message

    def _moveObjectClassesToSelected( self, REQUEST ):

        # A method for making an object class value "selected,"
        # rather than "available."  In addition to any specified
        # classes, be sure to move their superclasses, too.

        selectClasses = REQUEST.get('selectClasses',[])
        supers = []
        for item in selectClasses:
            supers = mergeLists( supers,
                                 getLDAPSuperClassNames( item,
                                                         self._ldapObjects ) 
                               )
        
        objectClassSchema = self._schema.getAttributeSchema( 'objectClass' )

        requiredValues = mergeLists( objectClassSchema.getRequiredValues(),
                                     selectClasses )

        # Now reassign any superclasses that also need to move

        supersToAdd = [item for item in supers 
                       if item not in requiredValues]
        if len( supersToAdd ) > 0:
            requiredValues = mergeLists( requiredValues, supersToAdd )
            message = "  Added superclasses to the required class list."

        objectClassSchema.setRequiredValues( requiredValues )

        message = self._saveAttributeSchema( REQUEST )

        return "Updated object class lists.  " + str(message)

    def _nextAttributeLink( self, REQUEST ):
        """
            Construct an href to the next managed attribute.
        """
        managed = self._schema.getManagedAttributeNames()
        ldapName = REQUEST.ldapName
        attributeIndex = managed.index( ldapName )

        if attributeIndex == len(managed)-1:
            return "&nbsp;"
        else:
            link = "<a href=\"%s?objectCategory=%s" % \
                   ( REQUEST['URL0'], REQUEST['objectCategory']) + \
                   "&schemaFormID=%s&ldapName=%s\">" % \
                   ( REQUEST['schemaFormID'], managed[ attributeIndex+1]) + \
                   "Next</a>"
            return link

    def _prevAttributeLink( self, REQUEST ):
        """
            Construct an href to the previous managed attribute.
        """
        managed = self._schema.getManagedAttributeNames()
        ldapName = REQUEST.ldapName
        attributeIndex = managed.index( ldapName )

        if attributeIndex == 0:
            return "&nbsp;"
        else:
            link = "<a href=\"%s?objectCategory=%s" % \
                   ( REQUEST['URL0'], REQUEST['objectCategory']) + \
                   "&schemaFormID=%s&ldapName=%s\">" % \
                   ( REQUEST['schemaFormID'], managed[ attributeIndex-1]) + \
                   "Previous</a>"
            return link

    def _renameDisplayCategories( self, REQUEST ):
        """
            Rename display categories.
        """
        #   If we got here from the _schemaRenameCategoriesForm, then 
        #   perform the rename.

        if REQUEST.has_key('haveNewNames'):
            ids = REQUEST.get( 'ids', [] )
            new_ids = REQUEST.get( 'new_ids', [] )
            displayCategories = self._schema.getDisplayCategories()

            # Skip empty new_ids

            while '' in new_ids:
                i = new_ids.index('')
                ids.pop(i)
                new_ids.pop(i)

            for i in range( 0, len( ids ) ):
                oldName = ids[ i ]
                newName = new_ids[ i ]

                #   Easy: change the list ...

                displayCategories[ displayCategories.index( oldName )] = \
                    newName

                #   Hard: change all of the attribute objects.

                for attributeSchema in self._schema._attributeCollection.values():
                    if oldName in attributeSchema.getDisplayCategories():
                        attributeSchema.delDisplayCategories( oldName )
                        attributeSchema.addDisplayCategories( newName )

            self._schema.setDisplayCategories( displayCategories )
            return "Renamed display categories"

        #   Otherwise, set REQUEST variables so that the
        #   _schemaRenameCategoriesForm gets displayed.

        else:
            REQUEST.set('schemaFormID', 'schemaRenameCategories')
            del REQUEST.form['updateSchema']

            return ""

    def _saveAttributeSchema( self, REQUEST ):
        """
            Save an updated attribute schema.
        """
        ldapName = REQUEST['ldapName']
        schema = self._schema.getAttributeSchema( ldapName )

        if not ldapName == 'objectClass':

            schema.setValidValues( REQUEST.get('validValues') )
            schema.setRequiredValues( REQUEST.get('requiredValues') )

        schema.setDisplayCategories( REQUEST.get('displayCategories') )
        schema.setFriendlyName( REQUEST.get('friendlyName', '') )
        schema.setManaged( REQUEST.get('managed', FALSE ) )
        schema.setPointsTo( REQUEST.get('pointsTo', '') )
        schema.setReaders( REQUEST.get('readers',[]) )
        schema.setRequired( REQUEST.get('required', FALSE ) )
        schema.setSyntaxByOID( REQUEST.get('syntax') )
        schema.setWriters( REQUEST.get('writers',[]) )

        return "Saved updated schema."

    def _saveAttrsByCategory( self, REQUEST ):
        """
            Save results of the schemaAttrsByCategory form.
        """
        displayCategory = REQUEST[ 'displayCategory' ]
        ids = REQUEST['ids']

        for ldapName in self._schema.getManagedAttributeNames():

            schema = self._schema.getAttributeSchema( ldapName )
            if ldapName in ids:
                schema.addDisplayCategories( displayCategory )
            elif displayCategory in schema.getDisplayCategories():
                schema.delDisplayCategories( displayCategory )

        return """
               Saved display category \"%s\"'s presence or absence
               in each managed attribute. 
               """ % ( displayCategory )

    def _saveAttrsByRole( self, REQUEST ):
        """
            Save results of the schemaAttrsByRole form.
        """
        roleName = REQUEST[ 'roleName' ]
        readIds = REQUEST['readIds']
        writeIds = REQUEST['writeIds']

        for ldapName in self._schema.getManagedAttributeNames():

            schema = self._schema.getAttributeSchema( ldapName )
            if ldapName in readIds:
                schema.addReaders( roleName )
            elif roleName in schema.getReaders():
                schema.delReaders( roleName )

            if ldapName in writeIds:
                schema.addWriters( roleName )
            elif roleName in schema.getWriters():
                schema.delWriters( roleName )

        return """
               Saved readers and writers settings for the %s role
               in each managed attribute.
               """ % ( roleName )

    def _setNamingAttribute( self, REQUEST ):

        # Set the naming attribute for this schema.

        namingAttribute = REQUEST.get( 'namingAttribute' )
        self._schema.setNamingAttribute( namingAttribute )

        return "Set naming attribute."

####################################################################
#
#   The method below is an auxiliary method that would be marked
#   as private class methods in other OO languages.  It does not
#   need any instance specific information and it should not be
#   "imported" into any other python module.
#
####################################################################

def getLDAPAttributeNames(objectClass, objectSchemas):
    """
        Returns lists of required and optional attributes for the
        given LDAP object class.
    """
    if not objectSchemas.has_key( objectClass.lower() ):
        return [],[]
    LDIFObject = objectSchemas[objectClass.lower()]
    required = LDIFObject.mustContain
    optional = LDIFObject.mayContain

    if LDIFObject.superClass:
        supRequired, supOptional = \
            getLDAPAttributeNames(LDIFObject.superClass, objectSchemas)
        required = mergeLists( required, supRequired )
        optional = mergeLists( optional, supOptional )

    return required, optional

def getLDAPSuperClassNames( objectClass, objectSchemas ):
    """
        Returns a list of superclass names for the given LDAP object
        class.
    """
    LDIFObject = objectSchemas[objectClass.lower()]

    if LDIFObject.superClass:
        super = [ LDIFObject.superClass, ]
        super.extend( getLDAPSuperClassNames( LDIFObject.superClass,
                                              objectSchemas ) )
        return super

    else:
        return []


