"""
    formSchemaManager.py
    $Author: payne $
    $Date: 2002/02/20 13:56:46 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    This class contains all of the methods that are in the
    STLDAPManager namespace (i.e., callable from DTML) pertaining
    to the Schema tab, used for modifying the Schema.

    Schema management is restricted to the Manager role.

    Schema management is split into three parts.  The schemas for
    'person', 'group', and 'orgUnit' types of object are managed
    separately.  The Schema tab provides a way to select which
    type of object schema is being managed.  The schema tab actually 
    consists of a number of forms,:

      o A form for editing the schema of individual managed attributes,
        allowing the manager to supply properties like a friendly name,
        and a list of valid values, and to specify which Zope roles
        have read and write access to the attribute.

      o A form for selecting the attributes to be managed from the list
        of all attributes known in the LDAP directory.

      o A form for selecting the objectClass list to be specified
        whenever a new object is created in the LDAP directory.
        
    The mutable schema object is the ZSchema object.  Displaying a
    Schema tab form consists of instantiating a ZSchema object with
    the schema for the type of object being considered, and storing
    it in the REQUEST.  The information required to draw the form
    can then be obtained by invoking methods of the ZSchema object,
    rather than methods of THIS object.

    Similarly, schema management, i.e., processing the result of
    submitting a Schema tab form, proceeds by instantiating a
    ZSchema object with the schema for the type of object being
    considered, modifying the ZSchema object as specified in
    the REQUEST, and saving the results back to the appropriate
    static schema object.

"""

__version__ = '$Revision: 1.29 $'[11:-2]

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
from Acquisition import Implicit
from Globals import DTMLFile, InitializeClass

from STLDAPUtilities import TRUE, FALSE
from ZSchema import ZSchema

class formSchemaManager( Implicit ):

    manage_options = ( { 'label'  : 'Schema',
                         'action' : 'manage_schema',
                         'help'   : ('STLDAPManager', 'Schema.stx' ) }, )

    security = ClassSecurityInfo()

    security.declareProtected('LDAP Manage Schema', 'manage_schema')
    security.declareProtected('LDAP Manage Schema', 'manage_schemaFormSelection')

    manage_schemaFormSelection = DTMLFile('dtml/schemaFormSelection', globals())

    #   DTML templates for the various "faces" of the Schema tab.
    #   Each template's forms must contain a hidden variable that
    #   identifies the form.  The name is 'schemaFormID', and the
    #   value must match one of the following after prepending an
    #   underscore, and appending 'Form', as in:
    #
    #     <input type=hidden name=schemaFormID value=schemaAttributes>
 
    _schemaAttributesForm = DTMLFile('dtml/schemaAttributes', globals())
    _schemaAttrsByCategoryForm = DTMLFile('dtml/schemaAttrsByCategory',
                                          globals())
    _schemaAttrsByRoleForm = DTMLFile('dtml/schemaAttrsByRole', globals())
    _schemaCategoriesForm = DTMLFile('dtml/schemaCategories', globals())
    _schemaControllersForm = DTMLFile('dtml/schemaControllers', globals())
    _schemaManagedForm = DTMLFile('dtml/schemaManaged', globals())
    _schemaObjectClassesForm = DTMLFile('dtml/schemaObjectClasses',
                                        globals())
    _schemaRenameCategoriesForm = DTMLFile('dtml/schemaRenameCategories', 
                                           globals())
    _schemaRenameRolesForm = DTMLFile('dtml/schemaRenameRoles', globals())
    _schemaRolesForm = DTMLFile('dtml/schemaRoles', globals())

    def manage_schema( self, REQUEST, message=None ):
        """
            Throw up the appropriate schema management form, depending
            on the activity.
        """
        if not REQUEST.has_key('schemaFormID'):
            REQUEST.set('schemaFormID','schemaObjectClasses')

        user = self._getUser( self.LDAPDirectory, REQUEST )
        zSchema = ZSchema( user, REQUEST )

        #   If we got here by a form submit button, then we need to
        #   update the schema first.

        if REQUEST.has_key('updateSchema'):
            if not REQUEST.has_key('cancel'):
                message = zSchema.updateSchemaFromRequest( REQUEST )
                self._p_changed = 1

                objectCategory = REQUEST['objectCategory']
                if objectCategory.lower() == 'person':
                    self._assignRolesToPermission( 'LDAP Manage People',
                                                   zSchema._schema )
                elif objectCategory.lower() == 'orgunit':
                    self._assignRolesToPermission( 'LDAP Manage Org Units',
                                                   zSchema._schema)
                elif objectCategory.lower() == 'group':
                    self._assignRolesToPermission( 'LDAP Manage Groups',
                                                   zSchema._schema )

                zSchema = ZSchema( user, REQUEST )

        #   Set default form, and render it.  Each form must contain a
        #   hidden tag called 'schemaFormID' whose value, with '_'
        #   prepended, and 'Form' appended, is form rendering DTMLFile
        #   call.

        if REQUEST.schemaFormID == 'schemaObjectClasses':
            return self._schemaObjectClassesForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaAttributes':
            return self._schemaAttributesForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaAttrsByCategory':
            return self._schemaAttrsByCategoryForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaAttrsByRole':
            return self._schemaAttrsByRoleForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaManaged':
            return self._schemaManagedForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaCategories':
            return self._schemaCategoriesForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaRenameCategories':
            return self._schemaRenameCategoriesForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaControllers':
            return self._schemaControllersForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaRoles':
            return self._schemaRolesForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.schemaFormID == 'schemaRenameRoles':
            return self._schemaRenameRolesForm( 
                        self, REQUEST, manage_tabs_message = message )

        else:
            message = """
                      The previous form's schemaFormID does not match a
                      known form display method.
                      """
            return self._schemaManagedForm( self, REQUEST,
                                            manage_tabs_message = message )

    def _assignRolesToPermission( self, permission, schema ):
        """
            Each of the three singleton ManagedSchema objects
            has a list of roles that can create/delete that
            category of objects.  Assign those roles to the
            specified permission.
        """
        roles = schema.getControllers()
        for role in roles:
            RoleManager._addRole( self, role )
        RoleManager.manage_permission( self, permission, roles, acquire=TRUE )

InitializeClass(formSchemaManager)    
